/*
 * This file is part of the nivo project.
 *
 * Copyright 2016-present, Raphaël Benitte.
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */
export var degreesToRadians = function degreesToRadians(degrees) {
    return degrees * Math.PI / 180;
};

export var radiansToDegrees = function radiansToDegrees(radians) {
    return 180 * radians / Math.PI;
};

/**
 * Try to get a neighbor arc, otherwise, returns null.
 *
 * @param {Number}   i
 * @param {function} identity
 * @param {Array}    prevData
 * @param {Array}    newData
 * @returns {{startAngle: *, endAngle: *}}
 */
export var findNeighbor = function findNeighbor(i, identity, prevData, newData) {
    var preceding = findPreceding(i, identity, prevData, newData);
    if (preceding) {
        return {
            startAngle: preceding.endAngle,
            endAngle: preceding.endAngle
        };
    }

    var following = findFollowing(i, identity, prevData, newData);
    if (following) {
        return {
            startAngle: following.startAngle,
            endAngle: following.startAngle
        };
    }

    return null;
};

/**
 * Find the element in prevData that joins the highest preceding element in newData.
 *
 * @param {Number}   i
 * @param {function} identity
 * @param {Array}    prevData
 * @param {Array}    newData
 * @returns {*}
 */
export var findPreceding = function findPreceding(i, identity, prevData, newData) {
    var m = prevData.length;

    while (--i >= 0) {
        var k = identity(newData[i]);

        for (var j = 0; j < m; ++j) {
            if (identity(prevData[j]) === k) {
                return prevData[j];
            }
        }
    }
};

/**
 * Find the element in prevData that joins the lowest following element in newData.
 *
 * @param {Number}   i
 * @param {function} identity
 * @param {Array}    prevData
 * @param {Array}    newData
 * @returns {*}
 */
export var findFollowing = function findFollowing(i, identity, prevData, newData) {
    var n = newData.length;
    var m = prevData.length;

    while (++i < n) {
        var k = identity(newData[i]);

        for (var j = 0; j < m; ++j) {
            if (identity(prevData[j]) === k) {
                return prevData[j];
            }
        }
    }
};

export var midAngle = function midAngle(arc) {
    return arc.startAngle + (arc.endAngle - arc.startAngle) / 2;
};

export var positionFromAngle = function positionFromAngle(angle, distance) {
    return {
        x: Math.cos(angle) * distance,
        y: Math.sin(angle) * distance
    };
};