function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

/*
 * This file is part of the nivo project.
 *
 * Copyright 2016-present, Raphaël Benitte.
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */
import React, { Component } from 'react';
import PropTypes from 'prop-types';
import { findDOMNode } from 'react-dom';
import d3 from 'd3';

var AxisX = function (_Component) {
    _inherits(AxisX, _Component);

    function AxisX() {
        _classCallCheck(this, AxisX);

        return _possibleConstructorReturn(this, _Component.apply(this, arguments));
    }

    AxisX.prototype.renderD3 = function renderD3(props) {
        var orient = props.orient,
            xScale = props.xScale,
            tickMode = props.tickMode,
            tickPadding = props.tickPadding,
            height = props.height,
            transitionDuration = props.transitionDuration,
            transitionEasing = props.transitionEasing;


        var element = d3.select(findDOMNode(this));
        element.attr('transform', 'translate(0, ' + height + ')');

        var axis = d3.svg.axis().scale(xScale).tickPadding(tickPadding).orient(orient);

        if (tickMode === 'grid') {
            axis.tickSize(-height);
        }

        element.transition().duration(transitionDuration).ease(transitionEasing).call(axis);

        return false;
    };

    AxisX.prototype.componentDidMount = function componentDidMount() {
        this.renderD3(this.props);
    };

    AxisX.prototype.shouldComponentUpdate = function shouldComponentUpdate(nextProps) {
        this.renderD3(nextProps);

        return false;
    };

    AxisX.prototype.render = function render() {
        return React.createElement('g', { className: 'chart__axis chart__axis--x' });
    };

    return AxisX;
}(Component);

AxisX.displayName = 'AxisX';

AxisX.propTypes = {
    orient: PropTypes.oneOf(['top', 'bottom']).isRequired,
    xScale: PropTypes.func.isRequired,
    tickMode: PropTypes.oneOf(['normal', 'grid']).isRequired,
    tickPadding: PropTypes.number.isRequired
};

AxisX.defaultProps = {
    orient: 'bottom',
    transitionDuration: 600,
    transitionEasing: 'cubic-out',
    tickMode: 'normal',
    tickPadding: 3
};

export default AxisX;