function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

/*
 * This file is part of the nivo project.
 *
 * Copyright 2016-present, Raphaël Benitte.
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */
import React, { Component } from 'react';
import PropTypes from 'prop-types';
import { findDOMNode } from 'react-dom';
import d3 from 'd3';

var AxisY = function (_Component) {
    _inherits(AxisY, _Component);

    function AxisY() {
        _classCallCheck(this, AxisY);

        return _possibleConstructorReturn(this, _Component.apply(this, arguments));
    }

    AxisY.prototype.renderD3 = function renderD3(props) {
        var yScale = props.yScale,
            tickCount = props.tickCount,
            tickFormat = props.tickFormat,
            orient = props.orient,
            tickMode = props.tickMode,
            tickPadding = props.tickPadding,
            width = props.width,
            transitionDuration = props.transitionDuration,
            transitionEasing = props.transitionEasing;


        var element = d3.select(findDOMNode(this));

        var axis = d3.svg.axis().scale(yScale).tickPadding(tickPadding).orient(orient).tickFormat(d3.format('s'));

        if (tickCount >= 0) {
            axis.ticks(tickCount);
        }

        if (tickFormat) {
            axis.tickFormat(d3.format(tickFormat));
        }

        if (tickMode === 'grid') {
            axis.tickSize(-width);
        }

        element.transition().duration(transitionDuration).ease(transitionEasing).call(axis);

        return false;
    };

    AxisY.prototype.componentDidMount = function componentDidMount() {
        this.renderD3(this.props);
    };

    AxisY.prototype.shouldComponentUpdate = function shouldComponentUpdate(nextProps) {
        this.renderD3(nextProps);

        return false;
    };

    AxisY.prototype.render = function render() {
        return React.createElement('g', { className: 'chart__axis chart__axis--y' });
    };

    return AxisY;
}(Component);

AxisY.displayName = 'AxisY';

AxisY.propTypes = {
    orient: PropTypes.oneOf(['left', 'right']).isRequired,
    yScale: PropTypes.func.isRequired,
    tickCount: PropTypes.number.isRequired,
    tickMode: PropTypes.oneOf(['normal', 'grid']).isRequired,
    tickPadding: PropTypes.number.isRequired
};

AxisY.defaultProps = {
    orient: 'left',
    transitionDuration: 600,
    transitionEasing: 'cubic-out',
    tickCount: -1,
    tickMode: 'normal',
    tickPadding: 3
};

export default AxisY;