var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

/*
 * This file is part of the nivo project.
 *
 * Copyright 2016-present, Raphaël Benitte.
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */
import React, { Component } from 'react';
import PropTypes from 'prop-types';
import _ from 'lodash-es';
import Nivo from '../../Nivo';
import { margin as marginPropType } from '../../PropTypes';
import Scale from '../scales/Scale';

var defaultTheme = {
    axis: {
        textColor: '#000',
        fontSize: '11px',
        tickColor: '#000',
        legendColor: '#000',
        legendFontSize: '11px'
    },
    grid: {
        stroke: '#000',
        strokeDasharray: '3,6'
    }
};

var Chart = function (_Component) {
    _inherits(Chart, _Component);

    function Chart() {
        _classCallCheck(this, Chart);

        return _possibleConstructorReturn(this, _Component.apply(this, arguments));
    }

    Chart.prototype.getChildContext = function getChildContext() {
        var theme = _.merge({}, defaultTheme, this.props.theme || {});

        return { nivoTheme: theme };
    };

    Chart.prototype.render = function render() {
        var _props = this.props,
            data = _props.data,
            children = _props.children,
            _margin = _props.margin,
            _width = _props.width,
            _height = _props.height,
            animate = _props.animate,
            motionStiffness = _props.motionStiffness,
            motionDamping = _props.motionDamping;


        var animationProps = {
            animate: animate,
            motionStiffness: motionStiffness,
            motionDamping: motionDamping
        };

        var margin = Object.assign({}, Nivo.defaults.margin, _margin);
        var width = _width - margin.left - margin.right;
        var height = _height - margin.top - margin.bottom;

        var scales = {};
        var axes = [];
        var items = [];

        React.Children.forEach(children, function (child, i) {
            if (child.type === Scale) {
                scales[child.props.id] = Scale.create(child.props, data, width, height);
            } else {
                var item = React.cloneElement(child, _extends({}, child.props, animationProps, {
                    key: i,
                    data: data,
                    scales: scales,
                    width: width,
                    height: height
                }));

                items.push(item);
            }
        });

        return React.createElement(
            'svg',
            { xmlns: 'http://www.w3.org/2000/svg', width: _width, height: _height },
            React.createElement(
                'g',
                { transform: 'translate(' + margin.left + ',' + margin.top + ')' },
                axes,
                items
            )
        );
    };

    return Chart;
}(Component);

Chart.propTypes = {
    // dimensions
    width: PropTypes.number.isRequired,
    height: PropTypes.number.isRequired,
    margin: marginPropType,
    // composition
    children: PropTypes.oneOfType([PropTypes.arrayOf(PropTypes.node), PropTypes.node]),
    // data
    data: PropTypes.arrayOf(PropTypes.object).isRequired,
    // motion
    animate: PropTypes.bool.isRequired,
    motionStiffness: PropTypes.number.isRequired,
    motionDamping: PropTypes.number.isRequired
};
Chart.defaultProps = {
    // dimensions
    margin: Nivo.defaults.margin,
    // motion
    animate: true,
    motionStiffness: Nivo.defaults.motionStiffness,
    motionDamping: Nivo.defaults.motionDamping
};
Chart.childContextTypes = {
    nivoTheme: PropTypes.object.isRequired
};
export default Chart;