var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

/*
 * This file is part of the nivo project.
 *
 * Copyright 2016-present, Raphaël Benitte.
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */
import React, { Component } from 'react';
import PropTypes from 'prop-types';
import { merge } from 'lodash-es';
import { TransitionMotion, spring } from 'react-motion';
import Nivo, { defaultTheme } from '../../../Nivo';
import { margin as marginPropType, motion as motionPropTypes } from '../../../PropTypes';
import { getColorRange } from '../../../ColorUtils';
import { generateGroupedBars, generateStackedBars } from '../../../lib/charts/bar';
import SvgWrapper from '../SvgWrapper';
import Axes from '../../axes/Axes';
import Grid from '../../axes/Grid';
import BarItem from './BarItem';
import BarItemLabel from './BarItemLabel';

var Bar = function (_Component) {
    _inherits(Bar, _Component);

    function Bar() {
        _classCallCheck(this, Bar);

        return _possibleConstructorReturn(this, _Component.apply(this, arguments));
    }

    Bar.prototype.render = function render() {
        var _props = this.props,
            data = _props.data,
            groupMode = _props.groupMode,
            _margin = _props.margin,
            _width = _props.width,
            _height = _props.height,
            colors = _props.colors,
            xPadding = _props.xPadding,
            axes = _props.axes,
            enableGridX = _props.enableGridX,
            enableGridY = _props.enableGridY,
            enableLabels = _props.enableLabels,
            _theme = _props.theme,
            animate = _props.animate,
            motionStiffness = _props.motionStiffness,
            motionDamping = _props.motionDamping;


        var margin = Object.assign({}, Nivo.defaults.margin, _margin);
        var width = _width - margin.left - margin.right;
        var height = _height - margin.top - margin.bottom;

        var theme = merge({}, defaultTheme, _theme);
        var color = getColorRange(colors);

        var motionProps = {
            animate: animate,
            motionDamping: motionDamping,
            motionStiffness: motionStiffness
        };

        var result = void 0;
        if (groupMode === 'grouped') {
            result = generateGroupedBars(data, width, height, color, {
                xPadding: xPadding
            });
        } else if (groupMode === 'stacked') {
            result = generateStackedBars(data, width, height, color, {
                xPadding: xPadding
            });
        }

        var bars = void 0;
        if (animate === true) {
            bars = React.createElement(
                TransitionMotion
                /*
                willEnter={this.willEnter}
                willLeave={this.willLeave}
                */
                ,
                { styles: result.bars.map(function (bar) {
                        return {
                            key: bar.key,
                            data: {
                                color: bar.color,
                                value: bar.value
                            },
                            style: {
                                x: spring(bar.x, motionProps),
                                y: spring(bar.y, motionProps),
                                width: spring(bar.width, motionProps),
                                height: spring(bar.height, motionProps)
                            }
                        };
                    })
                },
                function (interpolatedStyles) {
                    return React.createElement(
                        'g',
                        null,
                        interpolatedStyles.map(function (_ref) {
                            var key = _ref.key,
                                style = _ref.style,
                                _ref$data = _ref.data,
                                value = _ref$data.value,
                                color = _ref$data.color;
                            return React.createElement(BarItem, {
                                key: key,
                                x: style.x,
                                y: style.y,
                                width: style.width,
                                height: style.height,
                                color: color
                            });
                        })
                    );
                }
            );
        } else {
            bars = result.bars.map(function (d) {
                return React.createElement(BarItem, _extends({ key: d.key }, d));
            });
        }

        return React.createElement(
            SvgWrapper,
            { width: _width, height: _height, margin: margin },
            React.createElement(Grid, _extends({
                theme: theme,
                width: width,
                height: height,
                xScale: enableGridX ? result.xScale : null,
                yScale: enableGridY ? result.yScale : null
            }, motionProps)),
            React.createElement(Axes, _extends({
                axes: axes,
                xScale: result.xScale,
                yScale: result.yScale,
                width: width,
                height: height,
                theme: theme
            }, motionProps)),
            bars,
            enableLabels && result.bars.map(function (d) {
                return React.createElement(BarItemLabel, _extends({}, d, { key: d.key }));
            })
        );
    };

    return Bar;
}(Component);

Bar.propTypes = _extends({
    // data
    data: PropTypes.arrayOf(PropTypes.shape({
        id: PropTypes.string.isRequired,
        data: PropTypes.arrayOf(PropTypes.shape({
            x: PropTypes.oneOfType([PropTypes.number, PropTypes.string]).isRequired,
            y: PropTypes.oneOfType([PropTypes.number, PropTypes.string]).isRequired
        })).isRequired
    })).isRequired,

    groupMode: PropTypes.oneOf(['stacked', 'grouped']).isRequired,

    width: PropTypes.number.isRequired,
    height: PropTypes.number.isRequired,
    margin: marginPropType,
    xPadding: PropTypes.number.isRequired,

    // axes
    axes: PropTypes.object.isRequired,
    enableGridX: PropTypes.bool.isRequired,
    enableGridY: PropTypes.bool.isRequired,

    // labels
    enableLabels: PropTypes.bool.isRequired,

    // interactions
    onClick: PropTypes.func,

    theme: PropTypes.object.isRequired,
    colors: PropTypes.any.isRequired

}, motionPropTypes);
Bar.defaultProps = {
    margin: Nivo.defaults.margin,
    groupMode: 'stacked',
    colors: Nivo.defaults.colorRange,
    xPadding: 0.1,
    enableLabels: true,
    axes: {
        left: {
            enabled: true
        },
        bottom: {
            enabled: true
        }
    },
    enableGridX: false,
    enableGridY: true,
    theme: {},
    animate: true,
    motionStiffness: Nivo.defaults.motionStiffness,
    motionDamping: Nivo.defaults.motionDamping
};
export default Bar;