var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

/*
 * This file is part of the nivo project.
 *
 * Copyright 2016-present, Raphaël Benitte.
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */
import React, { Component } from 'react';
import { merge } from 'lodash-es';
import { TransitionMotion, spring } from 'react-motion';
import _ from 'lodash-es';
import Nivo, { defaultTheme } from '../../../Nivo';
import { getColorsGenerator, extractRGB } from '../../../lib/colorUtils';
import Container from '../Container';
import BubbleHelper from '../../../lib/charts/bubble/BubbleHelper';
import { convertGetter } from '../../../lib/propertiesConverters';
import { bubblePropTypes, bubbleDefaultProps } from './BubbleProps';

var ignoreProps = ['borderWidth', 'borderColor', 'enableLabel', 'label', 'labelFormat', 'labelTextColor', 'labelSkipRadius', 'labelTextDY', 'transitionDuration', 'transitionEasing'];

var BubblePlaceholders = function (_Component) {
    _inherits(BubblePlaceholders, _Component);

    function BubblePlaceholders() {
        _classCallCheck(this, BubblePlaceholders);

        return _possibleConstructorReturn(this, _Component.apply(this, arguments));
    }

    BubblePlaceholders.prototype.componentWillMount = function componentWillMount() {
        this.bubble = BubbleHelper();
    };

    BubblePlaceholders.prototype.willEnter = function willEnter(_ref) {
        var node = _ref.data;

        return _extends({
            r: 0,
            x: node.x,
            y: node.y
        }, extractRGB(node.color));
    };

    BubblePlaceholders.prototype.willLeave = function willLeave(styleThatLeft) {
        return {
            r: spring(0),
            x: spring(styleThatLeft.data.x),
            y: spring(styleThatLeft.data.y)
        };
    };

    BubblePlaceholders.prototype.render = function render() {
        var _this2 = this;

        var _props = this.props,
            root = _props.root,
            leavesOnly = _props.leavesOnly,
            namespace = _props.namespace,
            _width = _props.width,
            _height = _props.height,
            _identity = _props.identity,
            _value = _props.value,
            padding = _props.padding,
            _theme = _props.theme,
            colors = _props.colors,
            colorBy = _props.colorBy,
            animate = _props.animate,
            motionStiffness = _props.motionStiffness,
            motionDamping = _props.motionDamping,
            children = _props.children,
            isInteractive = _props.isInteractive;


        var identity = convertGetter(_identity);
        var value = convertGetter(_value);
        var color = getColorsGenerator(colors, colorBy);
        var theme = merge({}, defaultTheme, _theme);

        var margin = Object.assign({}, Nivo.defaults.margin, this.props.margin);
        var width = _width - margin.left - margin.right;
        var height = _height - margin.top - margin.bottom;

        var nodes = this.bubble.compute({
            width: width,
            height: height,
            root: root,
            leavesOnly: leavesOnly,
            identity: identity,
            value: value,
            padding: padding,
            color: color,
            colorBy: colorBy
        });

        var wrapperTag = void 0;
        var containerTag = void 0;

        var wrapperProps = {};
        var containerProps = {};

        if (namespace === 'svg') {
            wrapperTag = 'svg';
            containerTag = 'g';

            wrapperProps.width = _width;
            wrapperProps.height = _height;
            wrapperProps.xmlns = 'http://www.w3.org/2000/svg';
            containerProps.transform = 'translate(' + margin.left + ',' + margin.top + ')';
        } else {
            wrapperTag = 'div';
            containerTag = 'div';

            wrapperProps.style = {
                position: 'relative',
                width: _width,
                height: _height
            };
            containerProps.style = Object.assign({}, margin, {
                position: 'absolute'
            });
        }

        if (!animate) {
            return React.createElement(
                Container,
                { isInteractive: isInteractive, theme: theme },
                function (_ref2) {
                    var showTooltip = _ref2.showTooltip,
                        hideTooltip = _ref2.hideTooltip;
                    return React.createElement(wrapperTag, wrapperProps, React.createElement(containerTag, containerProps, children(nodes.map(function (node) {
                        return {
                            key: node.data.key,
                            data: node,
                            style: _.pick(node, ['r', 'x', 'y', 'color'])
                        };
                    }), { showTooltip: showTooltip, hideTooltip: hideTooltip })));
                }
            );
        }

        var motionProps = {
            stiffness: motionStiffness,
            damping: motionDamping
        };

        return React.createElement(
            Container,
            { isInteractive: isInteractive, theme: theme },
            function (_ref3) {
                var showTooltip = _ref3.showTooltip,
                    hideTooltip = _ref3.hideTooltip;
                return React.createElement(wrapperTag, wrapperProps, React.createElement(
                    TransitionMotion,
                    {
                        willEnter: _this2.willEnter,
                        willLeave: _this2.willLeave,
                        styles: nodes.map(function (b) {
                            return {
                                key: b.data.key,
                                data: b,
                                style: _extends({
                                    r: spring(b.r, motionProps),
                                    x: spring(b.x, motionProps),
                                    y: spring(b.y, motionProps)
                                }, extractRGB(b.color, motionProps))
                            };
                        })
                    },
                    function (interpolatedStyles) {
                        return React.createElement(containerTag, containerProps, children(interpolatedStyles.map(function (interpolatedStyle) {
                            var _interpolatedStyle$st = interpolatedStyle.style,
                                colorR = _interpolatedStyle$st.colorR,
                                colorG = _interpolatedStyle$st.colorG,
                                colorB = _interpolatedStyle$st.colorB;

                            interpolatedStyle.style.color = 'rgb(' + Math.round(colorR) + ',' + Math.round(colorG) + ',' + Math.round(colorB) + ')';

                            return interpolatedStyle;
                        }), { showTooltip: showTooltip, hideTooltip: hideTooltip }));
                    }
                ));
            }
        );
    };

    return BubblePlaceholders;
}(Component);

BubblePlaceholders.propTypes = _.omit(bubblePropTypes, ignoreProps);
BubblePlaceholders.defaultProps = _.omit(bubbleDefaultProps, ignoreProps);
export default BubblePlaceholders;