function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

/*
 * This file is part of the nivo project.
 *
 * Copyright 2016-present, Raphaël Benitte.
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */
import React, { Component } from 'react';
import { findDOMNode } from 'react-dom';
import d3 from 'd3';
import _ from 'lodash-es';
import Nivo from '../../../Nivo';
import CalendarLayout from '../../../lib/charts/calendar/CalendarLayout';
import { calendarPropTypes, calendarDefaultProps } from './CalendarProps';

var color = d3.scale.category20b();

var CalendarCanvas = function (_Component) {
    _inherits(CalendarCanvas, _Component);

    function CalendarCanvas(props) {
        _classCallCheck(this, CalendarCanvas);

        return _possibleConstructorReturn(this, _Component.call(this, props));
    }

    CalendarCanvas.prototype.renderD3 = function renderD3(props) {
        var from = props.from,
            to = props.to,
            direction = props.direction,
            yearSpacing = props.yearSpacing,
            daySpacing = props.daySpacing,
            dayBorderWidth = props.dayBorderWidth,
            dayBorderColor = props.dayBorderColor,
            monthBorderWidth = props.monthBorderWidth,
            monthBorderColor = props.monthBorderColor,
            transitionDuration = props.transitionDuration,
            transitionEasing = props.transitionEasing,
            transitionStaggering = props.transitionStaggering;


        var canvas = d3.select(findDOMNode(this));
        canvas.attr({
            width: props.width,
            height: props.height
        });

        var margin = _.assign({}, Nivo.defaults.margin, props.margin);
        var width = props.width - margin.left - margin.right;
        var height = props.height - margin.top - margin.bottom;

        var _calendarLayout$compu = this.calendarLayout.compute({
            width: width,
            height: height,
            from: from,
            to: to,
            direction: direction,
            yearSpacing: yearSpacing,
            daySpacing: daySpacing
        }),
            days = _calendarLayout$compu.days,
            months = _calendarLayout$compu.months;

        var dayNodes = this.container.selectAll('custom.rect').data(days, function (d) {
            return d.date;
        });

        dayNodes.enter().append('custom').classed('rect', true).attr('x', 0).attr('y', 0).attr('size', function (d) {
            return d.size;
        }).attr('fillStyle', function (d) {
            return color(d.date.getFullYear() + '.' + d.date.getMonth());
        }).attr('strokeStyle', dayBorderColor).attr('lineWidth', dayBorderWidth);

        dayNodes.transition().duration(transitionDuration).ease(transitionEasing).delay(function (d) {
            return d3.time.dayOfYear(d.date) * transitionStaggering;
        }).attr('x', function (d) {
            return d.x;
        }).attr('y', function (d) {
            return d.y;
        }).attr('size', function (d) {
            return d.size;
        }).attr('fillStyle', function (d) {
            return color(d.date.getFullYear() + '.' + d.date.getMonth());
        }).attr('strokeStyle', dayBorderColor).attr('lineWidth', dayBorderWidth);
    };

    CalendarCanvas.prototype.drawCanvas = function drawCanvas() {
        var ctx = this.context;

        var canvas = d3.select(findDOMNode(this));

        ctx.fillStyle = '#fff';
        ctx.rect(0, 0, canvas.attr('width'), canvas.attr('height'));
        ctx.fill();

        var days = this.container.selectAll('custom.rect');

        days.each(function (d) {
            var node = d3.select(this);

            ctx.beginPath();
            ctx.fillStyle = node.attr('fillStyle');
            ctx.strokeStyle = node.attr('strokeStyle');
            ctx.lineWidth = node.attr('lineWidth');
            ctx.rect(node.attr('x'), node.attr('y'), node.attr('size'), node.attr('size'));
            ctx.fill();
            ctx.stroke();
            ctx.closePath();
        });

        //console.log(days);
    };

    CalendarCanvas.prototype.componentWillMount = function componentWillMount() {
        this.calendarLayout = CalendarLayout();
        this.virtualContainer = document.createElement('custom');
        this.container = d3.select(this.virtualContainer);

        d3.timer(this.drawCanvas.bind(this));
    };

    CalendarCanvas.prototype.shouldComponentUpdate = function shouldComponentUpdate(nextProps, nextState) {
        this.renderD3(nextProps, nextState);

        return false;
    };

    CalendarCanvas.prototype.componentDidMount = function componentDidMount() {
        this.canvas = findDOMNode(this);
        this.context = this.canvas.getContext('2d');

        this.renderD3(this.props, this.state);
    };

    CalendarCanvas.prototype.render = function render() {
        return React.createElement('canvas', { className: 'nivo_calendar_canvas' });
    };

    return CalendarCanvas;
}(Component);

CalendarCanvas.propTypes = _.omit(calendarPropTypes, ['motionStiffness', 'motionDamping']);

CalendarCanvas.defaultProps = _.omit(calendarDefaultProps, ['motionStiffness', 'motionDamping']);

export default CalendarCanvas;