function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

/*
 * This file is part of the nivo project.
 *
 * Copyright 2016-present, Raphaël Benitte.
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */
import React, { Component } from 'react';
import { findDOMNode } from 'react-dom';
import d3 from 'd3';
import _ from 'lodash-es';
import Nivo from '../../../Nivo';
import CalendarLayout from '../../../lib/charts/calendar/CalendarLayout';
import { calendarPropTypes, calendarDefaultProps } from './CalendarProps';
import { DIRECTION_HORIZONTAL } from '../../../constants/directions';

var CalendarD3 = function (_Component) {
    _inherits(CalendarD3, _Component);

    function CalendarD3(props) {
        _classCallCheck(this, CalendarD3);

        return _possibleConstructorReturn(this, _Component.call(this, props));
    }

    CalendarD3.prototype.renderD3 = function renderD3(props) {
        var from = props.from,
            to = props.to,
            data = props.data,
            onDayClick = props.onDayClick,
            direction = props.direction,
            colorScale = props.colorScale,
            emptyColor = props.emptyColor,
            yearSpacing = props.yearSpacing,
            yearLegendOffset = props.yearLegendOffset,
            daySpacing = props.daySpacing,
            dayBorderWidth = props.dayBorderWidth,
            dayBorderColor = props.dayBorderColor,
            monthBorderWidth = props.monthBorderWidth,
            monthBorderColor = props.monthBorderColor,
            monthLegendOffset = props.monthLegendOffset,
            transitionDuration = props.transitionDuration,
            transitionEasing = props.transitionEasing,
            transitionStaggering = props.transitionStaggering;


        var element = d3.select(findDOMNode(this));
        var wrapper = element.select('.nivo_calendar_wrapper');

        var margin = _.assign({}, Nivo.defaults.margin, props.margin);
        var width = props.width - margin.left - margin.right;
        var height = props.height - margin.top - margin.bottom;

        element.select('.debug').attr('transform', 'translate(' + margin.left + ',' + margin.top + ')').attr({ width: width, height: height });

        element.attr({
            width: props.width,
            height: props.height
        });
        wrapper.attr('transform', 'translate(' + margin.left + ',' + margin.top + ')');

        var _calendarLayout$compu = this.calendarLayout.compute({
            width: width,
            height: height,
            from: from,
            to: to,
            data: data,
            direction: direction,
            colorScale: colorScale,
            emptyColor: emptyColor,
            yearSpacing: yearSpacing,
            daySpacing: daySpacing
        }),
            years = _calendarLayout$compu.years,
            months = _calendarLayout$compu.months,
            days = _calendarLayout$compu.days;

        // —————————————————————————————————————————————————————————————————————————————————————————————————————————————
        // Days
        // —————————————————————————————————————————————————————————————————————————————————————————————————————————————


        var dayNodes = wrapper.selectAll('.nivo_calendar_day').data(days, function (d) {
            return d.date;
        });

        dayNodes.enter().append('rect').attr('class', function (d) {
            return 'nivo_calendar_day nivo_calendar_day-month-' + d.date.getMonth();
        }).attr('width', function (d) {
            return d.size;
        }).attr('height', function (d) {
            return d.size;
        }).attr('x', 0).attr('y', 0).style({
            opacity: 0,
            fill: function fill(d) {
                return d.color;
            },
            stroke: dayBorderColor,
            'stroke-width': dayBorderWidth
        }).on('click', function (d) {
            return onDayClick(d);
        });

        dayNodes.transition().duration(transitionDuration).ease(transitionEasing).delay(function (d) {
            return d3.time.dayOfYear(d.date) * transitionStaggering;
        }).attr('width', function (d) {
            return d.size;
        }).attr('height', function (d) {
            return d.size;
        }).attr('x', function (d) {
            return d.x;
        }).attr('y', function (d) {
            return d.y;
        }).style({
            opacity: 1,
            fill: function fill(d) {
                return d.color;
            },
            stroke: dayBorderColor,
            'stroke-width': dayBorderWidth
        });

        // —————————————————————————————————————————————————————————————————————————————————————————————————————————————
        // Months
        // —————————————————————————————————————————————————————————————————————————————————————————————————————————————
        var monthPaths = wrapper.selectAll('.nivo_calendar_month').data(months, function (d) {
            return d.date;
        });

        monthPaths.enter().append('path').attr('class', 'nivo_calendar_month').style({
            opacity: 0,
            fill: 'none',
            stroke: monthBorderColor,
            'stroke-width': monthBorderWidth
        }).attr('d', function (d) {
            return d.path;
        });

        monthPaths.transition().duration(transitionDuration).ease(transitionEasing).delay(function (d) {
            return (d.date.getMonth() + 1) * 30 * transitionStaggering;
        }).style({
            opacity: 1,
            stroke: monthBorderColor,
            'stroke-width': monthBorderWidth
        }).attr('d', function (d) {
            return d.path;
        });

        // —————————————————————————————————————————————————————————————————————————————————————————————————————————————
        // Month legends
        // —————————————————————————————————————————————————————————————————————————————————————————————————————————————
        var monthLegends = wrapper.selectAll('.nivo_calendar_month_legend').data(months, function (d) {
            return d.date;
        });
        var monthLegendFormat = d3.time.format('%b');

        monthLegends.enter().append('text').classed('nivo_calendar_month_legend', true).text(function (d) {
            return monthLegendFormat(d.date);
        }).attr('text-anchor', 'middle').attr('transform', function (d) {
            if (direction === DIRECTION_HORIZONTAL) {
                return 'translate(' + (d.bbox.x + d.bbox.width / 2) + ',' + (d.bbox.y - monthLegendOffset) + ')';
            }

            return 'translate(' + (d.bbox.x - monthLegendOffset) + ',' + (d.bbox.y + d.bbox.height / 2) + ') rotate(-90)';
        }).style('opacity', 0);

        monthLegends.transition().duration(transitionDuration).ease(transitionEasing).delay(function (d) {
            return (d.date.getMonth() + 1) * 30 * transitionStaggering;
        }).attr('transform', function (d) {
            if (direction === DIRECTION_HORIZONTAL) {
                return 'translate(' + (d.bbox.x + d.bbox.width / 2) + ',' + (d.bbox.y - monthLegendOffset) + ')';
            }

            return 'translate(' + (d.bbox.x - monthLegendOffset) + ',' + (d.bbox.y + d.bbox.height / 2) + ') rotate(-90)';
        }).style('opacity', 1);

        // —————————————————————————————————————————————————————————————————————————————————————————————————————————————
        // Year legends
        // —————————————————————————————————————————————————————————————————————————————————————————————————————————————
        var yearLegends = wrapper.selectAll('.nivo_calendar_year_legend').data(years);

        var yearLabelRotation = direction === DIRECTION_HORIZONTAL ? -90 : 0;

        yearLegends.enter().append('text').text(function (d) {
            return d.year;
        }).classed('nivo_calendar_year_legend', true).attr('text-anchor', 'middle').attr('transform', function (d) {
            if (direction === DIRECTION_HORIZONTAL) {
                return 'translate(' + (d.bbox.x - yearLegendOffset) + ',' + (d.bbox.y + d.bbox.height / 2) + ') rotate(-90)';
            }

            return 'translate(' + (d.bbox.x + d.bbox.width / 2) + ',' + (d.bbox.y - yearLegendOffset) + ')';
        });

        yearLegends.transition().duration(transitionDuration).ease(transitionEasing).attr('transform', function (d) {
            if (direction === DIRECTION_HORIZONTAL) {
                return 'translate(' + (d.bbox.x - yearLegendOffset) + ',' + (d.bbox.y + d.bbox.height / 2) + ') rotate(-90)';
            }

            return 'translate(' + (d.bbox.x + d.bbox.width / 2) + ',' + (d.bbox.y - yearLegendOffset) + ')';
        });
    };

    CalendarD3.prototype.componentWillMount = function componentWillMount() {
        this.calendarLayout = CalendarLayout();
    };

    CalendarD3.prototype.shouldComponentUpdate = function shouldComponentUpdate(nextProps, nextState) {
        this.renderD3(nextProps, nextState);

        return false;
    };

    CalendarD3.prototype.componentDidMount = function componentDidMount() {
        this.renderD3(this.props, this.state);
    };

    CalendarD3.prototype.render = function render() {
        return React.createElement(
            'svg',
            { className: 'nivo_calendar' },
            React.createElement('g', { className: 'nivo_calendar_wrapper' })
        );
    };

    return CalendarD3;
}(Component);

CalendarD3.propTypes = _.omit(calendarPropTypes, ['enableMotion', 'motionStiffness', 'motionDamping']);

CalendarD3.defaultProps = _.omit(calendarDefaultProps, ['enableMotion', 'motionStiffness', 'motionDamping']);

export default CalendarD3;