var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

/*
 * This file is part of the nivo project.
 *
 * Copyright 2016-present, Raphaël Benitte.
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */
import React, { Component } from 'react';
import PropTypes from 'prop-types';
import { TransitionMotion, spring } from 'react-motion';
import { motionPropTypes } from '../../../props';
import { getLabelGenerator } from '../../../lib/propertiesConverters';
import MarkersItem from '../../markers/MarkersItem';

var LineMarkers = function (_Component) {
    _inherits(LineMarkers, _Component);

    function LineMarkers() {
        _classCallCheck(this, LineMarkers);

        return _possibleConstructorReturn(this, _Component.apply(this, arguments));
    }

    LineMarkers.prototype.render = function render() {
        var _props = this.props,
            lines = _props.lines,
            size = _props.size,
            color = _props.color,
            borderWidth = _props.borderWidth,
            borderColor = _props.borderColor,
            enableLabel = _props.enableLabel,
            label = _props.label,
            labelFormat = _props.labelFormat,
            labelYOffset = _props.labelYOffset,
            theme = _props.theme,
            animate = _props.animate,
            motionStiffness = _props.motionStiffness,
            motionDamping = _props.motionDamping;


        var getLabel = getLabelGenerator(label, labelFormat);

        var points = lines.reduce(function (acc, line) {
            var id = line.id,
                points = line.points;


            return [].concat(acc, points.map(function (point) {
                var pointData = {
                    serie: { id: id },
                    x: point.key,
                    y: point.value
                };

                return {
                    key: id + '.' + point.x,
                    x: point.x,
                    y: point.y,
                    fill: color(line),
                    stroke: borderColor(line),
                    label: enableLabel ? getLabel(pointData) : null
                };
            }));
        }, []);

        if (animate !== true) {
            return React.createElement(
                'g',
                null,
                points.map(function (point) {
                    return React.createElement(MarkersItem, {
                        key: point.key,
                        x: point.x,
                        y: point.y,
                        size: size,
                        color: point.fill,
                        borderWidth: borderWidth,
                        borderColor: point.stroke,
                        label: point.label,
                        labelYOffset: labelYOffset,
                        theme: theme
                    });
                })
            );
        }
        var springConfig = {
            motionDamping: motionDamping,
            motionStiffness: motionStiffness
        };

        return React.createElement(
            TransitionMotion,
            {
                styles: points.map(function (point) {
                    return {
                        key: point.key,
                        data: point,
                        style: {
                            x: spring(point.x, springConfig),
                            y: spring(point.y, springConfig),
                            size: spring(size, springConfig)
                        }
                    };
                })
            },
            function (interpolatedStyles) {
                return React.createElement(
                    'g',
                    null,
                    interpolatedStyles.map(function (_ref) {
                        var key = _ref.key,
                            style = _ref.style,
                            point = _ref.data;
                        return React.createElement(MarkersItem, _extends({
                            key: key
                        }, style, {
                            color: point.fill,
                            borderWidth: borderWidth,
                            borderColor: point.stroke,
                            label: point.label,
                            labelYOffset: labelYOffset,
                            theme: theme
                        }));
                    })
                );
            }
        );
    };

    return LineMarkers;
}(Component);

LineMarkers.propTypes = _extends({
    lines: PropTypes.arrayOf(PropTypes.shape({
        id: PropTypes.string.isRequired
    })),
    size: PropTypes.number.isRequired,
    color: PropTypes.func.isRequired,
    borderWidth: PropTypes.number.isRequired,
    borderColor: PropTypes.func.isRequired,

    // labels
    enableLabel: PropTypes.bool.isRequired,
    label: PropTypes.oneOfType([PropTypes.string, PropTypes.func]).isRequired,
    labelFormat: PropTypes.string,
    labelYOffset: PropTypes.number,

    // theming
    theme: PropTypes.shape({
        markers: PropTypes.shape({
            textColor: PropTypes.string.isRequired,
            fontSize: PropTypes.string.isRequired
        }).isRequired
    }).isRequired

}, motionPropTypes);
LineMarkers.defaultProps = {
    // labels
    enableLabel: false,
    label: 'y'
};
export default LineMarkers;