function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

/*
 * This file is part of the nivo project.
 *
 * Copyright 2016-present, Raphaël Benitte.
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */
import React, { Component } from 'react';
import PropTypes from 'prop-types';
import invariant from 'invariant';
import d3 from 'd3';
import Nivo from '../../../Nivo';
import { midAngle, radiansToDegrees } from '../../../lib/arcUtils';
import { getColorGenerator } from '../../../lib/colorUtils';

var PieRadialLegends = function (_Component) {
    _inherits(PieRadialLegends, _Component);

    function PieRadialLegends() {
        _classCallCheck(this, PieRadialLegends);

        return _possibleConstructorReturn(this, _Component.apply(this, arguments));
    }

    PieRadialLegends.decoratePie = function decoratePie(element) {
        var props = element.props;


        var color = getColorGenerator(props.textColor);

        return function (_ref) {
            var element = _ref.element,
                arc = _ref.arc,
                identity = _ref.identity,
                pie = _ref.pie,
                newData = _ref.newData,
                radius = _ref.radius;

            var labelFn = props.labelFn || identity;

            var outerArc = d3.svg.arc().innerRadius(radius + props.radiusOffset).outerRadius(radius + props.radiusOffset);

            var labels = element.selectAll('.radial-label').data(newData, identity);
            labels.enter().append('g').attr('class', 'radial-label').append('text').style('opacity', 0);

            labels.each(function (d) {
                var el = d3.select(this);

                var angle = midAngle(d);
                var angleOffset = angle < Math.PI ? -90 : 90;

                var styles = { opacity: 1 };
                if (color !== 'none') {
                    styles.fill = color(d);
                }

                el.select('text').text(labelFn).attr('text-anchor', function (d) {
                    return midAngle(d) < Math.PI ? 'start' : 'end';
                }).transition().duration(props.transitionDuration).ease(props.transitionEasing).style(styles).attr('transform', 'translate(' + (radius + props.radiusOffset) + ', 0)');
            }).transition().duration(props.transitionDuration).ease(props.transitionEasing).attr('transform', function (d) {
                var angle = midAngle(d);

                return 'rotate(' + radiansToDegrees(angle) + ', 0, 0)';
            });
            labels.exit().each(function (d) {
                var el = d3.select(this);

                el.select('text').transition().duration(props.transitionDuration).ease(props.transitionEasing).style('opacity', 0).attr('transform', 'translate(' + (radius + props.radiusOffset + 50) + ', 0)');
            }).transition().duration(0).delay(props.transitionDuration).remove();

            /*
            labels.enter()
                .append('text')
                .attr('class', 'radial-label')
            ;
            labels
                .text(labelFn)
                .attr('text-anchor', d => {
                    return midAngle(d) < Math.PI ? 'start' : 'end';
                })
                .transition()
                .duration(props.transitionDuration)
                .ease(props.transitionEasing)
                .attr('transform', d => {
                    const centroid = outerArc.centroid(d);
                    const angle    = midAngle(d);
                     const angleOffset = angle < Math.PI ? -90 : 90;
                     return `translate(${centroid[0]}, ${centroid[1]}) rotate(${radiansToDegrees(angle) + angleOffset}, 0, 0)`;
                })
            ;
            labels.exit()
                .transition()
                .duration(props.transitionDuration)
                .ease(props.transitionEasing)
                .style('opacity', 0)
                .remove()
            ;
            */
        };
    };

    PieRadialLegends.prototype.render = function render() {
        invariant(false, '<PieRadialLegends> element is for Pie configuration only and should not be rendered');
    };

    return PieRadialLegends;
}(Component);

var number = PropTypes.number,
    string = PropTypes.string,
    func = PropTypes.func,
    any = PropTypes.any;


PieRadialLegends.propTypes = {
    labelFn: func,
    radiusOffset: number.isRequired,
    transitionDuration: number.isRequired,
    transitionEasing: string.isRequired,
    textColor: any.isRequired
};

PieRadialLegends.defaultProps = {
    radiusOffset: 16,
    transitionDuration: Nivo.defaults.transitionDuration,
    transitionEasing: Nivo.defaults.transitionEasing,
    textColor: 'none'
};

export default PieRadialLegends;