function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

/*
 * This file is part of the nivo project.
 *
 * Copyright 2016-present, Raphaël Benitte.
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */
import React, { Component } from 'react';
import PropTypes from 'prop-types';
import { findDOMNode } from 'react-dom';
import _ from 'lodash-es';
import d3 from 'd3';
import Nivo from '../../../Nivo';
import { degreesToRadians, findNeighbor } from '../../../lib/arcUtils';
import { getColorRange } from '../../../lib/colorUtils';

var Pie = function (_Component) {
    _inherits(Pie, _Component);

    function Pie() {
        _classCallCheck(this, Pie);

        return _possibleConstructorReturn(this, _Component.apply(this, arguments));
    }

    Pie.prototype.renderD3 = function renderD3(nextProps) {
        var data = nextProps.data,
            width = nextProps.width,
            height = nextProps.height,
            sort = nextProps.sort,
            keyProp = nextProps.keyProp,
            valueProp = nextProps.valueProp,
            startAngle = nextProps.startAngle,
            endAngle = nextProps.endAngle,
            padAngle = nextProps.padAngle,
            colors = nextProps.colors,
            innerRadius = nextProps.innerRadius,
            transitionDuration = nextProps.transitionDuration,
            transitionEasing = nextProps.transitionEasing;


        var identity = function identity(d) {
            return d.data[keyProp];
        };

        var element = d3.select(findDOMNode(this));
        var container = element.select('.nivo_pie_slices');

        element.attr('transform', 'translate(' + width / 2 + ', ' + height / 2 + ')');

        var pie = d3.layout.pie().sort(sort).value(function (d) {
            return d[valueProp];
        }).startAngle(degreesToRadians(startAngle)).endAngle(degreesToRadians(endAngle)).padAngle(degreesToRadians(padAngle));

        var radius = Math.min(width / 2, height / 2);
        var arc = d3.svg.arc().outerRadius(radius).innerRadius(radius * innerRadius);

        var outline = element.select('.nivo_pie_outline').attr('d', arc({
            startAngle: degreesToRadians(startAngle),
            endAngle: degreesToRadians(endAngle)
        }));

        var color = getColorRange(colors);

        var slices = container.selectAll('.nivo_pie_slice');
        var previousData = slices.data();
        var newData = pie(data.map(function (d, i) {
            if (!d.color) {
                d.color = color(i);
            }

            return d;
        }));

        function arcTween(a) {
            var i = d3.interpolate(this._current, a);
            this._current = i(0);

            return function (t) {
                return arc(i(t));
            };
        }

        slices = slices.data(newData, identity);
        slices.enter().append('path').attr('class', 'nivo_pie_slice').style('fill', function (d) {
            return d.data.color;
        }).each(function (d, i) {
            this._current = findNeighbor(i, identity, previousData, newData) || _.assign({}, d, { endAngle: d.startAngle });
        });
        slices.exit().datum(function (d, i) {
            return findNeighbor(i, identity, newData, previousData) || d;
        }).transition().duration(transitionDuration).ease(transitionEasing).attrTween('d', arcTween).remove();
        slices.transition().duration(transitionDuration).ease(transitionEasing).attrTween('d', arcTween).style('fill', function (d) {
            return d.data.color;
        });

        var pieContext = {
            element: element,
            pie: pie,
            arc: arc,
            radius: radius,
            identity: identity,
            previousData: previousData,
            newData: newData
        };

        this.legends.forEach(function (legend) {
            legend(pieContext);
        });
    };

    Pie.prototype.shouldComponentUpdate = function shouldComponentUpdate(nextProps) {
        this.renderD3(nextProps);

        return false;
    };

    Pie.prototype.componentDidMount = function componentDidMount() {
        this.renderD3(this.props);
    };

    Pie.prototype.componentWillMount = function componentWillMount() {
        var children = this.props.children;


        var legends = [];

        React.Children.forEach(children, function (element) {
            if (React.isValidElement(element)) {
                if (element.type.createLegendsFromReactElement) {
                    legends.push(element.type.createLegendsFromReactElement(element));
                }
            }
        });

        this.legends = legends;
    };

    Pie.prototype.render = function render() {
        return React.createElement(
            'g',
            null,
            React.createElement('path', { className: 'nivo_pie_outline' }),
            React.createElement('g', { className: 'nivo_pie_slices' })
        );
    };

    return Pie;
}(Component);

var array = PropTypes.array,
    number = PropTypes.number,
    string = PropTypes.string,
    func = PropTypes.func,
    any = PropTypes.any;


Pie.propTypes = {
    width: number.isRequired,
    height: number.isRequired,
    sort: func,
    data: array.isRequired,
    keyProp: string.isRequired,
    valueProp: string.isRequired,
    startAngle: number.isRequired,
    endAngle: number.isRequired,
    padAngle: number.isRequired,
    colors: any.isRequired,
    transitionDuration: number.isRequired,
    transitionEasing: string.isRequired,
    innerRadius: number.isRequired
};

Pie.defaultProps = {
    sort: null,
    keyProp: 'label',
    valueProp: 'value',
    startAngle: 0,
    endAngle: 360,
    padAngle: 0,
    transitionDuration: Nivo.defaults.transitionDuration,
    transitionEasing: Nivo.defaults.transitionEasing,
    innerRadius: 0,
    colors: Nivo.defaults.colorRange
};

export default Pie;