function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

/*
 * This file is part of the nivo project.
 *
 * Copyright 2016-present, Raphaël Benitte.
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */

import React, { Component } from 'react';
import PropTypes from 'prop-types';
import { findDOMNode } from 'react-dom';
import _ from 'lodash-es';
import Nivo from '../../../Nivo';
import RadialStackD3Svg from '../../../lib/charts/radial-stack/RadialStackD3Svg';
import decoratorsFromReactChildren from '../../../lib/decoratorsFromReactChildren';
import { margin as marginPropType } from '../../../PropTypes';

var RADIAL_STACK_DECORATOR = 'decorateRadialStack';

var RadialStack = function (_Component) {
    _inherits(RadialStack, _Component);

    function RadialStack() {
        _classCallCheck(this, RadialStack);

        return _possibleConstructorReturn(this, _Component.apply(this, arguments));
    }

    RadialStack.prototype.shouldComponentUpdate = function shouldComponentUpdate(nextProps) {
        this.radialStack.decorate(decoratorsFromReactChildren(nextProps.children, RADIAL_STACK_DECORATOR));
        this.radialStack.draw(_.assign({}, nextProps, {
            margin: _.assign({}, Nivo.defaults.margin, nextProps.margin)
        }));

        return false;
    };

    RadialStack.prototype.componentDidMount = function componentDidMount() {
        this.radialStack = RadialStackD3Svg(findDOMNode(this));
        this.radialStack.decorate(decoratorsFromReactChildren(this.props.children, RADIAL_STACK_DECORATOR));
        this.radialStack.draw(_.assign({}, this.props, {
            margin: _.assign({}, Nivo.defaults.margin, this.props.margin)
        }));
    };

    RadialStack.prototype.render = function render() {
        return React.createElement('svg', { className: 'nivo_radial-stack' });
    };

    return RadialStack;
}(Component);

var array = PropTypes.array,
    number = PropTypes.number,
    string = PropTypes.string,
    func = PropTypes.func,
    any = PropTypes.any,
    oneOf = PropTypes.oneOf;


RadialStack.propTypes = {
    width: number.isRequired,
    height: number.isRequired,
    margin: marginPropType,
    innerRadius: number.isRequired,
    sort: func,
    layers: array.isRequired,
    offset: oneOf(['silhouette', 'wiggle', 'expand', 'zero']).isRequired,
    keyProp: string.isRequired,
    valueProp: string.isRequired,
    colors: any.isRequired,
    transitionDuration: number.isRequired,
    transitionEasing: string.isRequired
};

RadialStack.defaultProps = {
    margin: Nivo.defaults.margin,
    innerRadius: 0.6,
    sort: null,
    offset: 'zero',
    keyProp: 'label',
    valueProp: 'value',
    transitionDuration: Nivo.defaults.transitionDuration,
    transitionEasing: Nivo.defaults.transitionEasing,
    colors: Nivo.defaults.colorRange
};

export default RadialStack;