function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

/*
 * This file is part of the nivo project.
 *
 * Copyright 2016-present, Raphaël Benitte.
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */

import React, { Component } from 'react';
import PropTypes from 'prop-types';
import d3 from 'd3';
import invariant from 'invariant';
import { radiansToDegrees } from '../../../lib/ArcUtils';
import makeLabel, { LABEL_POSITION_TOP } from '../../../lib/charts/labels';

var RadialStackAngleAxis = function (_Component) {
    _inherits(RadialStackAngleAxis, _Component);

    function RadialStackAngleAxis() {
        _classCallCheck(this, RadialStackAngleAxis);

        return _possibleConstructorReturn(this, _Component.apply(this, arguments));
    }

    RadialStackAngleAxis.decorateRadialStack = function decorateRadialStack(_ref) {
        var _ref$props = _ref.props,
            labelPosition = _ref$props.labelPosition,
            labelRotation = _ref$props.labelRotation,
            labelOffset = _ref$props.labelOffset,
            labelPaddingX = _ref$props.labelPaddingX,
            labelPaddingY = _ref$props.labelPaddingY;

        var radialLine = d3.svg.line.radial().interpolate('cardinal-closed');
        /*
            .radius(radius)
            .angle(function(d, i) { return angle(i); });
        */
        return function (_ref2) {
            var element = _ref2.element,
                layers = _ref2.layers,
                stacked = _ref2.stacked,
                angle = _ref2.angle,
                radius = _ref2.radius,
                innerRadius = _ref2.innerRadius,
                outerRadius = _ref2.outerRadius,
                transitionDuration = _ref2.transitionDuration,
                transitionEasing = _ref2.transitionEasing;

            var wrapper = element.select('.nivo_radial-axis');
            if (wrapper.node() === null) {
                //wrapper = element.append('g')
                wrapper = element.insert('g', ':first-child').attr('class', 'nivo_radial-axis');
            }

            /*
            radialLine.angle(d => angle(d.x));
             const radialTicks = wrapper.selectAll('.nivo_radial-axis_circle').data([
                { radius: innerRadius, serie: stacked[0] },
                { radius: innerRadius + (outerRadius - innerRadius) / 2, serie: stacked[0] },
                { radius: outerRadius, serie: stacked[0] }
            ]);
             radialTicks.enter().append('path')
                .attr('class', 'nivo_radial-axis_circle')
                .attr('fill', 'none')
                .attr('stroke', '#000')
                .attr('d', d => radialLine.radius(d.radius)(d.serie))
            ;
            */

            var lines = wrapper.selectAll('.nivo_radial-axis_tick').data(stacked[0], function (d) {
                return d.x;
            });

            var newLine = lines.enter().append('g').attr('class', 'nivo_radial-axis_tick').attr('transform', 'rotate(-90)');

            newLine.append('line').attr('class', 'nivo_radial-axis_tick_grid-line').attr('x1', innerRadius).attr('x2', outerRadius);

            newLine.append('g').attr('transform', 'translate(' + outerRadius + ',0)').each(function (d) {
                var el = d3.select(this);

                el.append('g').attr('transform', function (d) {
                    return 'rotate(' + labelRotation + ')';
                }).call(makeLabel({
                    text: d.x,
                    position: labelPosition,
                    labelOffset: labelOffset,
                    labelPaddingX: labelPaddingX,
                    labelPaddingY: labelPaddingY
                }));
            });

            lines.transition().duration(transitionDuration).ease(transitionEasing).attr('transform', function (d) {
                return 'rotate(' + (radiansToDegrees(angle(d.x)) - 90) + ')';
            }).each(function (d) {
                var el = d3.select(this);

                d3.transition(el.select('line')).attr('x1', innerRadius).attr('x2', outerRadius);

                el.select('g').select('g').attr('transform', function (d) {
                    return 'rotate(' + labelRotation + ')';
                }).call(makeLabel({
                    text: d.x,
                    position: labelPosition,
                    labelOffset: labelOffset,
                    labelPaddingX: labelPaddingX,
                    labelPaddingY: labelPaddingY
                }));
            });
        };
    };

    RadialStackAngleAxis.prototype.render = function render() {
        invariant(false, '<RadialStackAngleAxis> element is for RadialStack components configuration only and should not be rendered');
    };

    return RadialStackAngleAxis;
}(Component);

var number = PropTypes.number,
    string = PropTypes.string;


RadialStackAngleAxis.propTypes = {
    labelPosition: string.isRequired,
    labelRotation: number.isRequired,
    labelOffset: number.isRequired,
    labelPaddingX: number.isRequired,
    labelPaddingY: number.isRequired
};

RadialStackAngleAxis.defaultProps = {
    labelPosition: LABEL_POSITION_TOP,
    labelRotation: 90,
    labelOffset: 10,
    labelPaddingX: 6,
    labelPaddingY: 2
};

export default RadialStackAngleAxis;