var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

/*
 * This file is part of the nivo project.
 *
 * Copyright 2016-present, Raphaël Benitte.
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */
import React from 'react';
import PropTypes from 'prop-types';
import { merge, isEqual } from 'lodash-es';
import { line } from 'd3-shape';
import compose from 'recompose/compose';
import pure from 'recompose/pure';
import withPropsOnChange from 'recompose/withPropsOnChange';
import defaultProps from 'recompose/defaultProps';
import Nivo, { defaultTheme } from '../../../Nivo';
import { marginPropType, motionPropTypes, curvePropMapping, curvePropType } from '../../../props';
import { getColorsGenerator, getInheritedColorGenerator } from '../../../lib/colorUtils';
import SvgWrapper from '../SvgWrapper';
import { getScales, getStackedScales, generateLines, generateStackedLines } from '../../../lib/charts/line';
import SmartMotion from '../../SmartMotion';
import Axes from '../../axes/Axes';
import Grid from '../../axes/Grid';
import LineMarkers from './LineMarkers';

var Line = function Line(_ref) {
    var data = _ref.data,
        stacked = _ref.stacked,
        lineGenerator = _ref.lineGenerator,
        xScale = _ref.xScale,
        yScale = _ref.yScale,
        margin = _ref.margin,
        width = _ref.width,
        height = _ref.height,
        fullWidth = _ref.fullWidth,
        fullHeight = _ref.fullHeight,
        axisTop = _ref.axisTop,
        axisRight = _ref.axisRight,
        axisBottom = _ref.axisBottom,
        axisLeft = _ref.axisLeft,
        enableGridX = _ref.enableGridX,
        enableGridY = _ref.enableGridY,
        enableMarkers = _ref.enableMarkers,
        markersSize = _ref.markersSize,
        markersColor = _ref.markersColor,
        markersBorderWidth = _ref.markersBorderWidth,
        markersBorderColor = _ref.markersBorderColor,
        enableMarkersLabel = _ref.enableMarkersLabel,
        markersLabel = _ref.markersLabel,
        markersLabelFormat = _ref.markersLabelFormat,
        markersLabelYOffset = _ref.markersLabelYOffset,
        theme = _ref.theme,
        color = _ref.color,
        animate = _ref.animate,
        motionStiffness = _ref.motionStiffness,
        motionDamping = _ref.motionDamping;

    var motionProps = {
        animate: animate,
        motionDamping: motionDamping,
        motionStiffness: motionStiffness
    };

    var lines = void 0;
    if (stacked === true) {
        lines = generateStackedLines(data, xScale, yScale, color);
    } else {
        lines = generateLines(data, xScale, yScale, color);
    }

    var markers = null;
    if (enableMarkers === true) {
        markers = React.createElement(LineMarkers, _extends({
            lines: lines,
            size: markersSize,
            color: getInheritedColorGenerator(markersColor),
            borderWidth: markersBorderWidth,
            borderColor: getInheritedColorGenerator(markersBorderColor),
            enableLabel: enableMarkersLabel,
            label: markersLabel,
            labelFormat: markersLabelFormat,
            labelYOffset: markersLabelYOffset,
            theme: theme
        }, motionProps));
    }

    var lineNodes = void 0;
    if (animate === true) {
        var springConfig = {
            stiffness: motionStiffness,
            damping: motionDamping
        };

        lineNodes = lines.map(function (_ref2) {
            var id = _ref2.id,
                lineColor = _ref2.color,
                points = _ref2.points;
            return React.createElement(
                SmartMotion,
                {
                    key: id,
                    style: function style(spring) {
                        return {
                            d: spring(lineGenerator(points), springConfig),
                            stroke: spring(lineColor, springConfig)
                        };
                    }
                },
                function (style) {
                    return React.createElement('path', { key: id, d: style.d, fill: 'none', strokeWidth: 2, stroke: style.stroke });
                }
            );
        });
    } else {
        lineNodes = lines.map(function (_ref3) {
            var id = _ref3.id,
                lineColor = _ref3.color,
                points = _ref3.points;
            return React.createElement('path', {
                key: id,
                d: lineGenerator(points),
                fill: 'none',
                strokeWidth: 2,
                stroke: lineColor
            });
        });
    }

    return React.createElement(
        SvgWrapper,
        { width: fullWidth, height: fullHeight, margin: margin },
        React.createElement(Grid, _extends({
            theme: theme,
            width: width,
            height: height,
            xScale: enableGridX ? xScale : null,
            yScale: enableGridY ? yScale : null
        }, motionProps)),
        React.createElement(Axes, _extends({
            xScale: xScale,
            yScale: yScale,
            width: width,
            height: height,
            theme: theme,
            top: axisTop,
            right: axisRight,
            bottom: axisBottom,
            left: axisLeft
        }, motionProps)),
        lineNodes,
        markers
    );
};

Line.propTypes = _extends({
    // data
    data: PropTypes.arrayOf(PropTypes.shape({
        id: PropTypes.string.isRequired,
        data: PropTypes.arrayOf(PropTypes.shape({
            x: PropTypes.oneOfType([PropTypes.number, PropTypes.string]).isRequired,
            y: PropTypes.oneOfType([PropTypes.number, PropTypes.string]).isRequired
        })).isRequired
    })).isRequired,

    stacked: PropTypes.bool.isRequired,
    curve: curvePropType.isRequired,
    lineGenerator: PropTypes.func.isRequired,

    xScale: PropTypes.func.isRequired,
    yScale: PropTypes.func.isRequired,

    // dimensions
    width: PropTypes.number.isRequired,
    height: PropTypes.number.isRequired,
    margin: marginPropType,

    // axes & grid
    axisTop: PropTypes.object,
    axisRight: PropTypes.object,
    axisBottom: PropTypes.object,
    axisLeft: PropTypes.object,
    enableGridX: PropTypes.bool.isRequired,
    enableGridY: PropTypes.bool.isRequired,

    // markers
    enableMarkers: PropTypes.bool.isRequired,
    markersSize: PropTypes.number.isRequired,
    markersColor: PropTypes.any.isRequired,
    markersBorderWidth: PropTypes.number.isRequired,
    markersBorderColor: PropTypes.any.isRequired,
    enableMarkersLabel: PropTypes.bool.isRequired,

    // theming
    theme: PropTypes.object.isRequired,
    colors: PropTypes.any.isRequired,
    colorBy: PropTypes.oneOfType([PropTypes.string, PropTypes.func]),
    color: PropTypes.func.isRequired

}, motionPropTypes);

export var LineDefaultProps = {
    stacked: false,
    curve: 'linear',

    // dimensions
    margin: Nivo.defaults.margin,

    // axes & grid
    axisBottom: {},
    axisLeft: {},
    enableGridX: true,
    enableGridY: true,

    // markers
    enableMarkers: true,
    markersSize: 6,
    markersColor: 'inherit',
    markersBorderWidth: 0,
    markersBorderColor: 'inherit',
    enableMarkersLabel: false,

    // theming
    theme: {},
    colors: 'nivo',
    colorBy: 'id',

    // motion
    animate: true,
    motionStiffness: Nivo.defaults.motionStiffness,
    motionDamping: Nivo.defaults.motionDamping
};

var enhance = compose(defaultProps(LineDefaultProps), withPropsOnChange(['theme'], function (_ref4) {
    var theme = _ref4.theme;
    return { theme: merge({}, defaultTheme, theme) };
}), withPropsOnChange(['curve'], function (_ref5) {
    var curve = _ref5.curve;
    return {
        lineGenerator: line().x(function (d) {
            return d.x;
        }).y(function (d) {
            return d.y;
        }).curve(curvePropMapping[curve])
    };
}), withPropsOnChange(['colors', 'colorBy'], function (_ref6) {
    var colors = _ref6.colors,
        colorBy = _ref6.colorBy;
    return {
        color: getColorsGenerator(colors, colorBy)
    };
}), withPropsOnChange(function (props, nextProps) {
    return props.data !== nextProps.data || props.stacked !== nextProps.stacked || props.width !== nextProps.width || props.height !== nextProps.height || !isEqual(props.margin, nextProps.margin);
}, function (_ref7) {
    var data = _ref7.data,
        stacked = _ref7.stacked,
        fullWidth = _ref7.width,
        fullHeight = _ref7.height,
        _margin = _ref7.margin;

    var margin = Object.assign({}, Nivo.defaults.margin, _margin);
    var width = fullWidth - margin.left - margin.right;
    var height = fullHeight - margin.top - margin.bottom;

    var scales = void 0;
    if (stacked === true) {
        scales = getStackedScales(data, width, height);
    } else {
        scales = getScales(data, width, height);
    }

    return _extends({
        margin: margin,
        width: width,
        height: height,
        fullWidth: fullWidth,
        fullHeight: fullHeight
    }, scales);
}), pure);

export default enhance(Line);