var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

/*
 * This file is part of the nivo project.
 *
 * Copyright 2016-present, Raphaël Benitte.
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */
import React, { Component } from 'react';
import { findDOMNode } from 'react-dom';
import _ from 'lodash-es';
import { TransitionMotion, spring } from 'react-motion';
import Nivo from '../../../Nivo';
import { computeTreeMap } from '../../../lib/charts/treemap/TreeMapHelper';
import { convertGetter } from '../../../lib/propertiesConverters';
import { treeMapPropTypes, treeMapDefaultProps } from './TreeMapProps';
import { getColorsGenerator, extractRGB } from '../../../lib/colorUtils';

var TreeMapPlaceholders = function (_Component) {
    _inherits(TreeMapPlaceholders, _Component);

    function TreeMapPlaceholders() {
        _classCallCheck(this, TreeMapPlaceholders);

        return _possibleConstructorReturn(this, _Component.apply(this, arguments));
    }

    TreeMapPlaceholders.prototype.nodeWillEnter = function nodeWillEnter(_ref) {
        var node = _ref.data;

        var width = node.x1 - node.x0;
        var height = node.y1 - node.y0;

        return _extends({
            x: node.x0 + width / 2,
            y: node.y0 + height / 2,
            width: 0,
            height: 0
        }, extractRGB(node.color));
    };

    TreeMapPlaceholders.prototype.render = function render() {
        var _this2 = this;

        var _props = this.props,
            root = _props.root,
            namespace = _props.namespace,
            _width = _props.width,
            _height = _props.height,
            tile = _props.tile,
            leavesOnly = _props.leavesOnly,
            innerPadding = _props.innerPadding,
            outerPadding = _props.outerPadding,
            _identity = _props.identity,
            _value = _props.value,
            animate = _props.animate,
            motionStiffness = _props.motionStiffness,
            motionDamping = _props.motionDamping,
            colors = _props.colors,
            colorBy = _props.colorBy;


        var identity = convertGetter(_identity);
        var value = convertGetter(_value);

        var margin = Object.assign({}, Nivo.defaults.margin, this.props.margin);
        var width = _width - margin.left - margin.right;
        var height = _height - margin.top - margin.bottom;

        var color = getColorsGenerator(colors, colorBy);

        var wrapperTag = void 0;
        var containerTag = void 0;

        var wrapperProps = {};
        var containerProps = {};

        if (namespace === 'svg') {
            wrapperTag = 'svg';
            containerTag = 'g';

            wrapperProps.width = _width;
            wrapperProps.height = _height;
            wrapperProps.xmlns = 'http://www.w3.org/2000/svg';
            containerProps.transform = 'translate(' + margin.left + ',' + margin.top + ')';
        } else {
            wrapperTag = 'div';
            containerTag = 'div';

            wrapperProps.style = {
                position: 'relative',
                width: _width,
                height: _height
            };
            containerProps.style = {
                position: 'absolute',
                top: margin.top,
                left: margin.left
            };
        }

        var nodes = computeTreeMap({
            width: width,
            height: height,
            root: root,
            tile: tile,
            leavesOnly: leavesOnly,
            innerPadding: innerPadding,
            outerPadding: outerPadding,
            identity: identity,
            value: value,
            color: color,
            colorBy: colorBy
        });

        if (animate === false) {
            return React.createElement(wrapperTag, wrapperProps, React.createElement(containerTag, containerProps, this.props.children(nodes.map(function (node) {
                return {
                    key: node.data.key,
                    data: node.data,
                    style: {
                        x: node.x0,
                        y: node.y0,
                        width: node.x1 - node.x0,
                        height: node.y1 - node.y0,
                        color: node.color
                    }
                };
            }))));
        }

        var springConfig = {
            stiffness: motionStiffness,
            damping: motionDamping
        };

        return React.createElement(wrapperTag, wrapperProps, React.createElement(
            TransitionMotion,
            {
                willEnter: this.nodeWillEnter,
                styles: nodes.map(function (node) {
                    return {
                        key: node.data.key,
                        data: node.data,
                        style: _extends({
                            x: spring(node.x0, springConfig),
                            y: spring(node.y0, springConfig),
                            width: spring(node.x1 - node.x0, springConfig),
                            height: spring(node.y1 - node.y0, springConfig)
                        }, extractRGB(node.color, springConfig))
                    };
                })
            },
            function (interpolatedStyles) {
                return React.createElement(containerTag, containerProps, _this2.props.children(interpolatedStyles.map(function (interpolatedStyle) {
                    var _interpolatedStyle$st = interpolatedStyle.style,
                        colorR = _interpolatedStyle$st.colorR,
                        colorG = _interpolatedStyle$st.colorG,
                        colorB = _interpolatedStyle$st.colorB;

                    interpolatedStyle.style.color = 'rgb(' + Math.round(colorR) + ',' + Math.round(colorG) + ',' + Math.round(colorB) + ')';

                    interpolatedStyle.style.width = Math.max(0, interpolatedStyle.style.width);
                    interpolatedStyle.style.height = Math.max(0, interpolatedStyle.style.height);

                    return interpolatedStyle;
                })));
            }
        ));
    };

    return TreeMapPlaceholders;
}(Component);

TreeMapPlaceholders.propTypes = _.omit(treeMapPropTypes, ['orientLabels', 'skipVMin', 'transitionDuration', 'transitionEasing']);
TreeMapPlaceholders.defaultProps = _.omit(treeMapDefaultProps, ['orientLabels', 'skipVMin', 'transitionDuration', 'transitionEasing']);
export default TreeMapPlaceholders;