function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

/*
 * This file is part of the nivo project.
 *
 * Copyright 2016-present, Raphaël Benitte.
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */

import React, { Component } from 'react';
import PropTypes from 'prop-types';
import { scaleLinear, scaleBand, scalePoint } from 'd3-scale';

var Scale = function (_Component) {
    _inherits(Scale, _Component);

    function Scale() {
        _classCallCheck(this, Scale);

        return _possibleConstructorReturn(this, _Component.apply(this, arguments));
    }

    Scale.create = function create(_ref, data, width, height) {
        var type = _ref.type,
            axis = _ref.axis,
            key = _ref.dataKey,
            aggregate = _ref.aggregate,
            maxOf = _ref.maxOf,
            padding = _ref.padding;

        var mapper = void 0;
        if (Array.isArray(aggregate)) {
            mapper = function mapper(d) {
                return _.sum(aggregate.map(function (k) {
                    return d[k];
                }));
            };
        } else if (Array.isArray(maxOf)) {
            mapper = function mapper(d) {
                return _.max(maxOf.map(function (k) {
                    return d[k];
                }));
            };
        } else if (_.isFunction(key)) {
            mapper = key;
        } else {
            mapper = function mapper(d) {
                return d[key];
            };
        }

        var range = [axis === 'y' ? height : 0, axis === 'x' ? width : 0];

        var scale = void 0;
        switch (type) {
            case 'linear':
                scale = scaleLinear().rangeRound(range).domain([0, _.max(data.map(mapper))]);
                break;

            case 'band':
                scale = scaleBand().rangeRound(range).domain(data.map(mapper));

                if (padding !== undefined) {
                    scale.padding(padding);
                }
                break;

            case 'point':
                scale = scalePoint().range(range).domain(data.map(mapper));
                break;
        }

        return scale;
    };

    Scale.prototype.render = function render() {
        return null;
    };

    return Scale;
}(Component);

Scale.propTypes = {
    id: PropTypes.string.isRequired,
    type: PropTypes.oneOf(['linear', 'point', 'band']).isRequired,
    padding: PropTypes.number,
    dataKey: PropTypes.oneOfType([PropTypes.string, PropTypes.number, PropTypes.func]),
    aggregate: PropTypes.array,
    maxOf: PropTypes.array
};
export default Scale;