function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

/*
 * This file is part of the nivo project.
 *
 * Copyright 2016-present, Raphaël Benitte.
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */

import React, { Component } from 'react';
import PropTypes from 'prop-types';
import d3 from 'd3';

var XYScales = function (_Component) {
    _inherits(XYScales, _Component);

    function XYScales() {
        _classCallCheck(this, XYScales);

        return _possibleConstructorReturn(this, _Component.apply(this, arguments));
    }

    XYScales.prototype.render = function render() {
        var _props = this.props,
            data = _props.data,
            identityAccessor = _props.identityAccessor,
            valueAccessor = _props.valueAccessor,
            width = _props.width,
            height = _props.height,
            children = _props.children;


        if (React.Children.count(children) === 0) {
            return null;
        }

        var xScale = d3.scale.linear().range([0, width]).domain([0, data.length - 1]);

        var yScale = d3.scale.linear().range([height, 0]).domain([0, d3.max(data.map(valueAccessor))]);

        return React.createElement(
            'g',
            null,
            React.Children.map(children, function (child) {
                return React.cloneElement(child, {
                    data: data.slice(),
                    xScale: xScale,
                    yScale: yScale,
                    width: width,
                    height: height
                });
            })
        );
    };

    return XYScales;
}(Component);

XYScales.displayName = 'XYScales';

var array = PropTypes.array,
    func = PropTypes.func,
    number = PropTypes.number;


XYScales.propTypes = {
    data: array.isRequired,
    identityAccessor: func.isRequired,
    valueAccessor: func.isRequired,
    width: number,
    height: number
};

XYScales.defaultProps = {
    identityAccessor: function identityAccessor(d, i) {
        return i;
    },
    valueAccessor: function valueAccessor(d) {
        return d;
    }
};

export default XYScales;