function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

/*
 * This file is part of the nivo project.
 *
 * Copyright 2016-present, Raphaël Benitte.
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */

import React, { Component } from 'react';
import PropTypes from 'prop-types';
import { findDOMNode } from 'react-dom';
import d3 from 'd3';
import Nivo from '../../Nivo';

var Area = function (_Component) {
    _inherits(Area, _Component);

    function Area() {
        _classCallCheck(this, Area);

        return _possibleConstructorReturn(this, _Component.apply(this, arguments));
    }

    Area.prototype.renderD3 = function renderD3(props) {
        var data = props.data,
            xScale = props.xScale,
            yScale = props.yScale,
            xAccessor = props.xAccessor,
            yAccessor = props.yAccessor,
            width = props.width,
            height = props.height,
            interpolation = props.interpolation,
            transitionDuration = props.transitionDuration,
            transitionEasing = props.transitionEasing;


        var element = d3.select(findDOMNode(this));

        var area = d3.svg.area().x(function (d, i) {
            return xScale(xAccessor(d, i));
        }).y0(height).y1(function (d, i) {
            return yScale(yAccessor(d, i));
        }).interpolate(interpolation);

        element.datum(data).transition().duration(transitionDuration).ease(transitionEasing).attr('d', area);
    };

    Area.prototype.componentDidMount = function componentDidMount() {
        this.renderD3(this.props);
    };

    Area.prototype.shouldComponentUpdate = function shouldComponentUpdate(nextProps) {
        this.renderD3(nextProps);

        return false;
    };

    Area.prototype.render = function render() {
        return React.createElement('path', { className: 'chart__shape__area' });
    };

    return Area;
}(Component);

Area.displayName = 'Area';

var array = PropTypes.array,
    func = PropTypes.func,
    string = PropTypes.string,
    number = PropTypes.number;


Area.propTypes = {
    data: array.isRequired,
    xScale: func.isRequired,
    yScale: func.isRequired,
    xAccessor: func.isRequired,
    yAccessor: func.isRequired,
    width: number.isRequired,
    height: number.isRequired,
    interpolation: string.isRequired,
    transitionDuration: number.isRequired,
    transitionEasing: string.isRequired
};

Area.defaultProps = {
    xAccessor: function xAccessor(d, i) {
        return i;
    },
    yAccessor: function yAccessor(d) {
        return d;
    },
    interpolation: 'linear',
    transitionDuration: Nivo.defaults.transitionDuration,
    transitionEasing: Nivo.defaults.transitionEasing
};

export default Area;