/*
 * This file is part of the nivo project.
 *
 * Copyright 2016-present, Raphaël Benitte.
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */

import _ from 'lodash-es';
import { format } from 'd3-format';

export var getLabelGenerator = function getLabelGenerator(_label, labelFormat) {
    if (_.isFunction(_label)) {
        return _label;
    }

    var label = function label(d) {
        return _.get(d, _label);
    };

    var formatter = void 0;
    if (labelFormat) {
        formatter = format(labelFormat);
    }

    return function (data) {
        var labelOutput = label(data);

        if (formatter) {
            labelOutput = formatter(labelOutput);
        }

        return labelOutput;
    };
};

export var convertGetter = function convertGetter(_property) {
    if (_.isFunction(_property)) {
        return _property;
    }

    return function (d) {
        return _.get(d, _property);
    };
};

export var getAccessorFor = function getAccessorFor(directive) {
    return _.isFunction(directive) ? directive : function (d) {
        return d[directive];
    };
};