'use strict';

exports.__esModule = true;
exports.motion = exports.margin = exports.scale = exports.lineInterpolation = undefined;

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var number = _propTypes2.default.number,
    string = _propTypes2.default.string,
    array = _propTypes2.default.array,
    shape = _propTypes2.default.shape,
    oneOf = _propTypes2.default.oneOf; /*
                                        * This file is part of the nivo project.
                                        *
                                        * Copyright 2016-present, Raphaël Benitte.
                                        *
                                        * For the full copyright and license information, please view the LICENSE
                                        * file that was distributed with this source code.
                                        */

var lineInterpolation = exports.lineInterpolation = oneOf(['linear', // piecewise linear segments, as in a polyline.
'linear-closed', // close the linear segments to form a polygon.
'step', // alternate between horizontal and vertical segments, as in a step function.
'step-before', // alternate between vertical and horizontal segments, as in a step function.
'step-after', // alternate between horizontal and vertical segments, as in a step function.
'basis', // a B-spline, with control point duplication on the ends.
'basis-open', // an open B-spline; may not intersect the start or end.
'basis-closed', // a closed B-spline, as in a loop.
'bundle', // equivalent to basis, except the tension parameter is used to straighten the spline.
'cardinal', // a Cardinal spline, with control point duplication on the ends.
'cardinal-open', // an open Cardinal spline; may not intersect the start or end, but will intersect other control points.
'cardinal-closed', // a closed Cardinal spline, as in a loop.
'monotone'] // cubic interpolation that preserves monotonicity in y.
);

var scale = exports.scale = shape({
    type: string.isRequired,
    domain: array.isRequired,
    range: array.isRequired
});

var margin = exports.margin = shape({
    top: number,
    right: number,
    bottom: number,
    left: number
}).isRequired;

var motion = exports.motion = {
    animate: _propTypes2.default.bool.isRequired,
    motionStiffness: _propTypes2.default.number.isRequired,
    motionDamping: _propTypes2.default.number.isRequired
};