'use strict';

exports.__esModule = true;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _Nivo = require('../../Nivo');

var _Nivo2 = _interopRequireDefault(_Nivo);

var _PropTypes = require('../../PropTypes');

var _Scale = require('../scales/Scale');

var _Scale2 = _interopRequireDefault(_Scale);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; } /*
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                * This file is part of the nivo project.
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                *
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                * Copyright 2016-present, Raphaël Benitte.
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                *
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                * For the full copyright and license information, please view the LICENSE
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                * file that was distributed with this source code.
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                */


var defaultTheme = {
    axis: {
        textColor: '#000',
        fontSize: '11px',
        tickColor: '#000',
        legendColor: '#000',
        legendFontSize: '11px'
    },
    grid: {
        stroke: '#000',
        strokeDasharray: '3,6'
    }
};

var Chart = function (_Component) {
    _inherits(Chart, _Component);

    function Chart() {
        _classCallCheck(this, Chart);

        return _possibleConstructorReturn(this, _Component.apply(this, arguments));
    }

    Chart.prototype.getChildContext = function getChildContext() {
        var theme = _lodash2.default.merge({}, defaultTheme, this.props.theme || {});

        return { nivoTheme: theme };
    };

    Chart.prototype.render = function render() {
        var _props = this.props,
            data = _props.data,
            children = _props.children,
            _margin = _props.margin,
            _width = _props.width,
            _height = _props.height,
            animate = _props.animate,
            motionStiffness = _props.motionStiffness,
            motionDamping = _props.motionDamping;


        var animationProps = {
            animate: animate,
            motionStiffness: motionStiffness,
            motionDamping: motionDamping
        };

        var margin = Object.assign({}, _Nivo2.default.defaults.margin, _margin);
        var width = _width - margin.left - margin.right;
        var height = _height - margin.top - margin.bottom;

        var scales = {};
        var axes = [];
        var items = [];

        _react2.default.Children.forEach(children, function (child, i) {
            if (child.type === _Scale2.default) {
                scales[child.props.id] = _Scale2.default.create(child.props, data, width, height);
            } else {
                var item = _react2.default.cloneElement(child, _extends({}, child.props, animationProps, {
                    key: i,
                    data: data,
                    scales: scales,
                    width: width,
                    height: height
                }));

                items.push(item);
            }
        });

        return _react2.default.createElement(
            'svg',
            { xmlns: 'http://www.w3.org/2000/svg', width: _width, height: _height },
            _react2.default.createElement(
                'g',
                { transform: 'translate(' + margin.left + ',' + margin.top + ')' },
                axes,
                items
            )
        );
    };

    return Chart;
}(_react.Component);

Chart.propTypes = {
    // dimensions
    width: _propTypes2.default.number.isRequired,
    height: _propTypes2.default.number.isRequired,
    margin: _PropTypes.margin,
    // composition
    children: _propTypes2.default.oneOfType([_propTypes2.default.arrayOf(_propTypes2.default.node), _propTypes2.default.node]),
    // data
    data: _propTypes2.default.arrayOf(_propTypes2.default.object).isRequired,
    // motion
    animate: _propTypes2.default.bool.isRequired,
    motionStiffness: _propTypes2.default.number.isRequired,
    motionDamping: _propTypes2.default.number.isRequired
};
Chart.defaultProps = {
    // dimensions
    margin: _Nivo2.default.defaults.margin,
    // motion
    animate: true,
    motionStiffness: _Nivo2.default.defaults.motionStiffness,
    motionDamping: _Nivo2.default.defaults.motionDamping
};
Chart.childContextTypes = {
    nivoTheme: _propTypes2.default.object.isRequired
};
exports.default = Chart;