'use strict';

exports.__esModule = true;
exports.BarDefaultProps = undefined;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; /*
                                                                                                                                                                                                                                                                   * This file is part of the nivo project.
                                                                                                                                                                                                                                                                   *
                                                                                                                                                                                                                                                                   * Copyright 2016-present, Raphaël Benitte.
                                                                                                                                                                                                                                                                   *
                                                                                                                                                                                                                                                                   * For the full copyright and license information, please view the LICENSE
                                                                                                                                                                                                                                                                   * file that was distributed with this source code.
                                                                                                                                                                                                                                                                   */


var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

var _lodash = require('lodash');

var _reactMotion = require('react-motion');

var _compose = require('recompose/compose');

var _compose2 = _interopRequireDefault(_compose);

var _defaultProps = require('recompose/defaultProps');

var _defaultProps2 = _interopRequireDefault(_defaultProps);

var _withPropsOnChange = require('recompose/withPropsOnChange');

var _withPropsOnChange2 = _interopRequireDefault(_withPropsOnChange);

var _pure = require('recompose/pure');

var _pure2 = _interopRequireDefault(_pure);

var _hocs = require('../../../hocs');

var _colorUtils = require('../../../lib/colorUtils');

var _bar = require('../../../lib/charts/bar');

var _propertiesConverters = require('../../../lib/propertiesConverters');

var _Container = require('../Container');

var _Container2 = _interopRequireDefault(_Container);

var _SvgWrapper = require('../SvgWrapper');

var _SvgWrapper2 = _interopRequireDefault(_SvgWrapper);

var _Axes = require('../../axes/Axes');

var _Axes2 = _interopRequireDefault(_Axes);

var _Grid = require('../../axes/Grid');

var _Grid2 = _interopRequireDefault(_Grid);

var _BarItem = require('./BarItem');

var _BarItem2 = _interopRequireDefault(_BarItem);

var _BarItemLabel = require('./BarItemLabel');

var _BarItemLabel2 = _interopRequireDefault(_BarItemLabel);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var Bar = function Bar(_ref) {
    var data = _ref.data,
        getIndex = _ref.getIndex,
        keys = _ref.keys,
        groupMode = _ref.groupMode,
        layout = _ref.layout,
        margin = _ref.margin,
        width = _ref.width,
        height = _ref.height,
        outerWidth = _ref.outerWidth,
        outerHeight = _ref.outerHeight,
        xPadding = _ref.xPadding,
        axisTop = _ref.axisTop,
        axisRight = _ref.axisRight,
        axisBottom = _ref.axisBottom,
        axisLeft = _ref.axisLeft,
        enableGridX = _ref.enableGridX,
        enableGridY = _ref.enableGridY,
        enableLabels = _ref.enableLabels,
        getLabelsLinkColor = _ref.getLabelsLinkColor,
        getLabelsTextColor = _ref.getLabelsTextColor,
        theme = _ref.theme,
        getColor = _ref.getColor,
        animate = _ref.animate,
        motionStiffness = _ref.motionStiffness,
        motionDamping = _ref.motionDamping,
        isInteractive = _ref.isInteractive;

    var motionProps = {
        animate: animate,
        motionDamping: motionDamping,
        motionStiffness: motionStiffness
    };

    var result = void 0;
    if (groupMode === 'grouped') {
        result = (0, _bar.generateGroupedBars)(layout, data, getIndex, keys, width, height, getColor, {
            xPadding: xPadding
        });
    } else if (groupMode === 'stacked') {
        result = (0, _bar.generateStackedBars)(layout, data, getIndex, keys, width, height, getColor, {
            xPadding: xPadding
        });
    }

    return _react2.default.createElement(
        _Container2.default,
        { isInteractive: isInteractive },
        function (_ref2) {
            var showTooltip = _ref2.showTooltip,
                hideTooltip = _ref2.hideTooltip;

            var bars = void 0;
            if (animate === true) {
                bars = _react2.default.createElement(
                    _reactMotion.TransitionMotion,
                    {
                        styles: result.bars.map(function (bar) {
                            return {
                                key: bar.key,
                                data: bar,
                                style: {
                                    x: (0, _reactMotion.spring)(bar.x, motionProps),
                                    y: (0, _reactMotion.spring)(bar.y, motionProps),
                                    width: (0, _reactMotion.spring)(bar.width, motionProps),
                                    height: (0, _reactMotion.spring)(bar.height, motionProps)
                                }
                            };
                        })
                    },
                    function (interpolatedStyles) {
                        return _react2.default.createElement(
                            'g',
                            null,
                            interpolatedStyles.map(function (_ref3) {
                                var key = _ref3.key,
                                    style = _ref3.style,
                                    data = _ref3.data;
                                return _react2.default.createElement(_BarItem2.default, _extends({
                                    key: key
                                }, data, style, {
                                    showTooltip: showTooltip,
                                    hideTooltip: hideTooltip
                                }));
                            })
                        );
                    }
                );
            } else {
                bars = result.bars.map(function (d) {
                    return _react2.default.createElement(_BarItem2.default, _extends({
                        key: d.key
                    }, d, {
                        showTooltip: showTooltip,
                        hideTooltip: hideTooltip
                    }));
                });
            }

            return _react2.default.createElement(
                _SvgWrapper2.default,
                { width: outerWidth, height: outerHeight, margin: margin },
                _react2.default.createElement(_Grid2.default, _extends({
                    theme: theme,
                    width: width,
                    height: height,
                    xScale: enableGridX ? result.xScale : null,
                    yScale: enableGridY ? result.yScale : null
                }, motionProps)),
                _react2.default.createElement(_Axes2.default, _extends({
                    xScale: result.xScale,
                    yScale: result.yScale,
                    width: width,
                    height: height,
                    theme: theme,
                    top: axisTop,
                    right: axisRight,
                    bottom: axisBottom,
                    left: axisLeft
                }, motionProps)),
                bars,
                enableLabels && result.bars.map(function (d) {
                    return _react2.default.createElement(_BarItemLabel2.default, _extends({}, d, {
                        textColor: getLabelsTextColor(d, theme),
                        linkColor: getLabelsLinkColor(d, theme)
                    }));
                })
            );
        }
    );
};

Bar.propTypes = {
    // data
    data: _propTypes2.default.arrayOf(_propTypes2.default.object).isRequired,
    indexBy: _propTypes2.default.oneOfType([_propTypes2.default.string, _propTypes2.default.func]).isRequired,
    getIndex: _propTypes2.default.func.isRequired, // computed
    keys: _propTypes2.default.arrayOf(_propTypes2.default.string).isRequired,

    groupMode: _propTypes2.default.oneOf(['stacked', 'grouped']).isRequired,
    layout: _propTypes2.default.oneOf(['horizontal', 'vertical']).isRequired,

    xPadding: _propTypes2.default.number.isRequired,

    // axes & grid
    axisTop: _propTypes2.default.object,
    axisRight: _propTypes2.default.object,
    axisBottom: _propTypes2.default.object,
    axisLeft: _propTypes2.default.object,
    enableGridX: _propTypes2.default.bool.isRequired,
    enableGridY: _propTypes2.default.bool.isRequired,

    // labels
    enableLabels: _propTypes2.default.bool.isRequired,
    labelsTextColor: _propTypes2.default.oneOfType([_propTypes2.default.string, _propTypes2.default.func]).isRequired,
    getLabelsTextColor: _propTypes2.default.func.isRequired, // computed
    labelsLinkColor: _propTypes2.default.oneOfType([_propTypes2.default.string, _propTypes2.default.func]).isRequired,
    getLabelsLinkColor: _propTypes2.default.func.isRequired, // computed

    // interactions
    onClick: _propTypes2.default.func,

    // theming
    getColor: _propTypes2.default.func.isRequired,

    // interactivity
    isInteractive: _propTypes2.default.bool
};

var BarDefaultProps = exports.BarDefaultProps = {
    indexBy: 'id',
    keys: ['value'],

    groupMode: 'stacked',
    layout: 'vertical',

    xPadding: 0.1,

    // axes & grid
    axisBottom: {},
    axisLeft: {},
    enableGridX: false,
    enableGridY: true,

    // labels
    enableLabels: true,
    labelsLinkColor: 'theme',
    labelsTextColor: 'theme',

    // interactivity
    isInteractive: true
};

Bar.defaultProps = BarDefaultProps;

var enhance = (0, _compose2.default)((0, _defaultProps2.default)(BarDefaultProps), (0, _hocs.withTheme)(), (0, _hocs.withColors)(), (0, _hocs.withDimensions)(), (0, _hocs.withMotion)(), (0, _withPropsOnChange2.default)(['indexBy'], function (_ref4) {
    var indexBy = _ref4.indexBy;
    return {
        getIndex: (0, _propertiesConverters.getAccessorFor)(indexBy)
    };
}), (0, _withPropsOnChange2.default)(['labelsTextColor'], function (_ref5) {
    var labelsTextColor = _ref5.labelsTextColor;
    return {
        getLabelsTextColor: (0, _colorUtils.getInheritedColorGenerator)(labelsTextColor, 'axis.textColor')
    };
}), (0, _withPropsOnChange2.default)(['labelsLinkColor'], function (_ref6) {
    var labelsLinkColor = _ref6.labelsLinkColor;
    return {
        getLabelsLinkColor: (0, _colorUtils.getInheritedColorGenerator)(labelsLinkColor, 'axis.tickColor')
    };
}), _pure2.default);

exports.default = enhance(Bar);