'use strict';

exports.__esModule = true;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

var _lodash = require('lodash');

var _reactMotion = require('react-motion');

var _Nivo = require('../../../Nivo');

var _Nivo2 = _interopRequireDefault(_Nivo);

var _PropTypes = require('../../../PropTypes');

var _ColorUtils = require('../../../ColorUtils');

var _bar = require('../../../lib/charts/bar');

var _SvgWrapper = require('../SvgWrapper');

var _SvgWrapper2 = _interopRequireDefault(_SvgWrapper);

var _Axes = require('../../axes/Axes');

var _Axes2 = _interopRequireDefault(_Axes);

var _Grid = require('../../axes/Grid');

var _Grid2 = _interopRequireDefault(_Grid);

var _BarItem = require('./BarItem');

var _BarItem2 = _interopRequireDefault(_BarItem);

var _BarItemLabel = require('./BarItemLabel');

var _BarItemLabel2 = _interopRequireDefault(_BarItemLabel);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; } /*
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                * This file is part of the nivo project.
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                *
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                * Copyright 2016-present, Raphaël Benitte.
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                *
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                * For the full copyright and license information, please view the LICENSE
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                * file that was distributed with this source code.
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                */


var Bar = function (_Component) {
    _inherits(Bar, _Component);

    function Bar() {
        _classCallCheck(this, Bar);

        return _possibleConstructorReturn(this, _Component.apply(this, arguments));
    }

    Bar.prototype.render = function render() {
        var _props = this.props,
            data = _props.data,
            groupMode = _props.groupMode,
            _margin = _props.margin,
            _width = _props.width,
            _height = _props.height,
            colors = _props.colors,
            xPadding = _props.xPadding,
            axes = _props.axes,
            enableGridX = _props.enableGridX,
            enableGridY = _props.enableGridY,
            enableLabels = _props.enableLabels,
            _theme = _props.theme,
            animate = _props.animate,
            motionStiffness = _props.motionStiffness,
            motionDamping = _props.motionDamping;


        var margin = Object.assign({}, _Nivo2.default.defaults.margin, _margin);
        var width = _width - margin.left - margin.right;
        var height = _height - margin.top - margin.bottom;

        var theme = (0, _lodash.merge)({}, _Nivo.defaultTheme, _theme);
        var color = (0, _ColorUtils.getColorRange)(colors);

        var motionProps = {
            animate: animate,
            motionDamping: motionDamping,
            motionStiffness: motionStiffness
        };

        var result = void 0;
        if (groupMode === 'grouped') {
            result = (0, _bar.generateGroupedBars)(data, width, height, color, {
                xPadding: xPadding
            });
        } else if (groupMode === 'stacked') {
            result = (0, _bar.generateStackedBars)(data, width, height, color, {
                xPadding: xPadding
            });
        }

        var bars = void 0;
        if (animate === true) {
            bars = _react2.default.createElement(
                _reactMotion.TransitionMotion
                /*
                willEnter={this.willEnter}
                willLeave={this.willLeave}
                */
                ,
                { styles: result.bars.map(function (bar) {
                        return {
                            key: bar.key,
                            data: {
                                color: bar.color,
                                value: bar.value
                            },
                            style: {
                                x: (0, _reactMotion.spring)(bar.x, motionProps),
                                y: (0, _reactMotion.spring)(bar.y, motionProps),
                                width: (0, _reactMotion.spring)(bar.width, motionProps),
                                height: (0, _reactMotion.spring)(bar.height, motionProps)
                            }
                        };
                    })
                },
                function (interpolatedStyles) {
                    return _react2.default.createElement(
                        'g',
                        null,
                        interpolatedStyles.map(function (_ref) {
                            var key = _ref.key,
                                style = _ref.style,
                                _ref$data = _ref.data,
                                value = _ref$data.value,
                                color = _ref$data.color;
                            return _react2.default.createElement(_BarItem2.default, {
                                key: key,
                                x: style.x,
                                y: style.y,
                                width: style.width,
                                height: style.height,
                                color: color
                            });
                        })
                    );
                }
            );
        } else {
            bars = result.bars.map(function (d) {
                return _react2.default.createElement(_BarItem2.default, _extends({ key: d.key }, d));
            });
        }

        return _react2.default.createElement(
            _SvgWrapper2.default,
            { width: _width, height: _height, margin: margin },
            _react2.default.createElement(_Grid2.default, _extends({
                theme: theme,
                width: width,
                height: height,
                xScale: enableGridX ? result.xScale : null,
                yScale: enableGridY ? result.yScale : null
            }, motionProps)),
            _react2.default.createElement(_Axes2.default, _extends({
                axes: axes,
                xScale: result.xScale,
                yScale: result.yScale,
                width: width,
                height: height,
                theme: theme
            }, motionProps)),
            bars,
            enableLabels && result.bars.map(function (d) {
                return _react2.default.createElement(_BarItemLabel2.default, _extends({}, d, { key: d.key }));
            })
        );
    };

    return Bar;
}(_react.Component);

Bar.propTypes = _extends({
    // data
    data: _propTypes2.default.arrayOf(_propTypes2.default.shape({
        id: _propTypes2.default.string.isRequired,
        data: _propTypes2.default.arrayOf(_propTypes2.default.shape({
            x: _propTypes2.default.oneOfType([_propTypes2.default.number, _propTypes2.default.string]).isRequired,
            y: _propTypes2.default.oneOfType([_propTypes2.default.number, _propTypes2.default.string]).isRequired
        })).isRequired
    })).isRequired,

    groupMode: _propTypes2.default.oneOf(['stacked', 'grouped']).isRequired,

    width: _propTypes2.default.number.isRequired,
    height: _propTypes2.default.number.isRequired,
    margin: _PropTypes.margin,
    xPadding: _propTypes2.default.number.isRequired,

    // axes
    axes: _propTypes2.default.object.isRequired,
    enableGridX: _propTypes2.default.bool.isRequired,
    enableGridY: _propTypes2.default.bool.isRequired,

    // labels
    enableLabels: _propTypes2.default.bool.isRequired,

    // interactions
    onClick: _propTypes2.default.func,

    theme: _propTypes2.default.object.isRequired,
    colors: _propTypes2.default.any.isRequired

}, _PropTypes.motion);
Bar.defaultProps = {
    margin: _Nivo2.default.defaults.margin,
    groupMode: 'stacked',
    colors: _Nivo2.default.defaults.colorRange,
    xPadding: 0.1,
    enableLabels: true,
    axes: {
        left: {
            enabled: true
        },
        bottom: {
            enabled: true
        }
    },
    enableGridX: false,
    enableGridY: true,
    theme: {},
    animate: true,
    motionStiffness: _Nivo2.default.defaults.motionStiffness,
    motionDamping: _Nivo2.default.defaults.motionDamping
};
exports.default = Bar;