'use strict';

exports.__esModule = true;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _propertiesConverters = require('../../../lib/propertiesConverters');

var _colorUtils = require('../../../lib/colorUtils');

var _BubbleProps = require('./BubbleProps');

var _BubblePlaceholders = require('./BubblePlaceholders');

var _BubblePlaceholders2 = _interopRequireDefault(_BubblePlaceholders);

var _BasicTooltip = require('../../tooltip/BasicTooltip');

var _BasicTooltip2 = _interopRequireDefault(_BasicTooltip);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; } /*
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                * This file is part of the nivo project.
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                *
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                * Copyright 2016-present, Raphaël Benitte.
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                *
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                * For the full copyright and license information, please view the LICENSE
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                * file that was distributed with this source code.
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                */


var createNodes = function createNodes(_ref) {
    var borderWidth = _ref.borderWidth,
        borderColor = _ref.borderColor,
        enableLabel = _ref.enableLabel,
        _label = _ref.label,
        labelFormat = _ref.labelFormat,
        labelSkipRadius = _ref.labelSkipRadius,
        labelTextColor = _ref.labelTextColor,
        labelTextDY = _ref.labelTextDY;

    var label = (0, _propertiesConverters.getLabelGenerator)(_label, labelFormat);
    var borderColorFn = (0, _colorUtils.getColorGenerator)(borderColor);
    var textColorFn = (0, _colorUtils.getColorGenerator)(labelTextColor);

    return function (nodes, _ref2) {
        var showTooltip = _ref2.showTooltip,
            hideTooltip = _ref2.hideTooltip;

        var renderedNodes = [];

        nodes.filter(function (node) {
            return node.style.r > 0;
        }).forEach(function (node) {
            var handleTooltip = function handleTooltip(e) {
                return showTooltip(_react2.default.createElement(_BasicTooltip2.default, {
                    id: node.data.data.name,
                    value: node.data.value,
                    enableChip: true,
                    color: node.style.color
                }), e);
            };

            renderedNodes.push(_react2.default.createElement('circle', {
                key: node.key + '.circle',
                r: node.style.r,
                className: 'nivo_bubble_node',
                transform: 'translate(' + node.style.x + ',' + node.style.y + ')',
                onMouseEnter: handleTooltip,
                onMouseMove: handleTooltip,
                onMouseLeave: hideTooltip,
                style: {
                    fill: node.style.color,
                    stroke: borderColorFn(node.style),
                    strokeWidth: borderWidth
                }
            }));
        });

        if (enableLabel === true) {
            nodes.filter(function (node) {
                return node.data.height === 0 && (labelSkipRadius === 0 || node.data.r >= labelSkipRadius);
            }).forEach(function (node) {
                renderedNodes.push(_react2.default.createElement(
                    'text',
                    {
                        key: node.key + '.text',
                        className: 'nivo_bubble_legend',
                        transform: 'translate(' + node.style.x + ',' + node.style.y + ')',
                        textAnchor: 'middle',
                        dy: labelTextDY,
                        style: {
                            fill: textColorFn(node.style),
                            pointerEvents: 'none'
                        }
                    },
                    label(node.data.data)
                ));
            });
        }

        return renderedNodes;
    };
};

var Bubble = function (_Component) {
    _inherits(Bubble, _Component);

    function Bubble() {
        _classCallCheck(this, Bubble);

        return _possibleConstructorReturn(this, _Component.apply(this, arguments));
    }

    Bubble.prototype.render = function render() {
        return _react2.default.createElement(
            _BubblePlaceholders2.default,
            _extends({}, this.props, { namespace: 'svg' }),
            createNodes(this.props)
        );
    };

    return Bubble;
}(_react.Component);

Bubble.propTypes = _lodash2.default.omit(_BubbleProps.bubblePropTypes, ['children', 'namespace', 'transitionDuration', 'transitionEasing']);
Bubble.defaultProps = _lodash2.default.omit(_BubbleProps.bubbleDefaultProps, ['namespace', 'transitionDuration', 'transitionEasing']);
exports.default = Bubble;