'use strict';

exports.__esModule = true;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _reactMotion = require('react-motion');

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _Nivo = require('../../../Nivo');

var _Nivo2 = _interopRequireDefault(_Nivo);

var _colorUtils = require('../../../lib/colorUtils');

var _Container = require('../Container');

var _Container2 = _interopRequireDefault(_Container);

var _BubbleHelper = require('../../../lib/charts/bubble/BubbleHelper');

var _BubbleHelper2 = _interopRequireDefault(_BubbleHelper);

var _propertiesConverters = require('../../../lib/propertiesConverters');

var _BubbleProps = require('./BubbleProps');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; } /*
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                * This file is part of the nivo project.
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                *
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                * Copyright 2016-present, Raphaël Benitte.
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                *
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                * For the full copyright and license information, please view the LICENSE
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                * file that was distributed with this source code.
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                */


var ignoreProps = ['borderWidth', 'borderColor', 'enableLabel', 'label', 'labelFormat', 'labelTextColor', 'labelSkipRadius', 'labelTextDY', 'transitionDuration', 'transitionEasing'];

var BubblePlaceholders = function (_Component) {
    _inherits(BubblePlaceholders, _Component);

    function BubblePlaceholders() {
        _classCallCheck(this, BubblePlaceholders);

        return _possibleConstructorReturn(this, _Component.apply(this, arguments));
    }

    BubblePlaceholders.prototype.componentWillMount = function componentWillMount() {
        this.bubble = (0, _BubbleHelper2.default)();
    };

    BubblePlaceholders.prototype.willEnter = function willEnter(_ref) {
        var node = _ref.data;

        return _extends({
            r: 0,
            x: node.x,
            y: node.y
        }, (0, _colorUtils.extractRGB)(node.color));
    };

    BubblePlaceholders.prototype.willLeave = function willLeave(styleThatLeft) {
        return {
            r: (0, _reactMotion.spring)(0),
            x: (0, _reactMotion.spring)(styleThatLeft.data.x),
            y: (0, _reactMotion.spring)(styleThatLeft.data.y)
        };
    };

    BubblePlaceholders.prototype.render = function render() {
        var _this2 = this;

        var _props = this.props,
            root = _props.root,
            leavesOnly = _props.leavesOnly,
            namespace = _props.namespace,
            _width = _props.width,
            _height = _props.height,
            _identity = _props.identity,
            _value = _props.value,
            padding = _props.padding,
            colors = _props.colors,
            colorBy = _props.colorBy,
            animate = _props.animate,
            motionStiffness = _props.motionStiffness,
            motionDamping = _props.motionDamping,
            children = _props.children,
            isInteractive = _props.isInteractive;


        var identity = (0, _propertiesConverters.convertGetter)(_identity);
        var value = (0, _propertiesConverters.convertGetter)(_value);
        var color = (0, _colorUtils.getColorsGenerator)(colors, colorBy);

        var margin = Object.assign({}, _Nivo2.default.defaults.margin, this.props.margin);
        var width = _width - margin.left - margin.right;
        var height = _height - margin.top - margin.bottom;

        var nodes = this.bubble.compute({
            width: width,
            height: height,
            root: root,
            leavesOnly: leavesOnly,
            identity: identity,
            value: value,
            padding: padding,
            color: color,
            colorBy: colorBy
        });

        var wrapperTag = void 0;
        var containerTag = void 0;

        var wrapperProps = {};
        var containerProps = {};

        if (namespace === 'svg') {
            wrapperTag = 'svg';
            containerTag = 'g';

            wrapperProps.width = _width;
            wrapperProps.height = _height;
            wrapperProps.xmlns = 'http://www.w3.org/2000/svg';
            containerProps.transform = 'translate(' + margin.left + ',' + margin.top + ')';
        } else {
            wrapperTag = 'div';
            containerTag = 'div';

            wrapperProps.style = {
                position: 'relative',
                width: _width,
                height: _height
            };
            containerProps.style = Object.assign({}, margin, {
                position: 'absolute'
            });
        }

        if (!animate) {
            return _react2.default.createElement(
                _Container2.default,
                { isInteractive: isInteractive },
                function (_ref2) {
                    var showTooltip = _ref2.showTooltip,
                        hideTooltip = _ref2.hideTooltip;
                    return _react2.default.createElement(wrapperTag, wrapperProps, _react2.default.createElement(containerTag, containerProps, children(nodes.map(function (node) {
                        return {
                            key: node.data.key,
                            data: node,
                            style: _lodash2.default.pick(node, ['r', 'x', 'y', 'color'])
                        };
                    }), { showTooltip: showTooltip, hideTooltip: hideTooltip })));
                }
            );
        }

        var motionProps = {
            stiffness: motionStiffness,
            damping: motionDamping
        };

        return _react2.default.createElement(
            _Container2.default,
            { isInteractive: isInteractive },
            function (_ref3) {
                var showTooltip = _ref3.showTooltip,
                    hideTooltip = _ref3.hideTooltip;
                return _react2.default.createElement(wrapperTag, wrapperProps, _react2.default.createElement(
                    _reactMotion.TransitionMotion,
                    {
                        willEnter: _this2.willEnter,
                        willLeave: _this2.willLeave,
                        styles: nodes.map(function (b) {
                            return {
                                key: b.data.key,
                                data: b,
                                style: _extends({
                                    r: (0, _reactMotion.spring)(b.r, motionProps),
                                    x: (0, _reactMotion.spring)(b.x, motionProps),
                                    y: (0, _reactMotion.spring)(b.y, motionProps)
                                }, (0, _colorUtils.extractRGB)(b.color, motionProps))
                            };
                        })
                    },
                    function (interpolatedStyles) {
                        return _react2.default.createElement(containerTag, containerProps, children(interpolatedStyles.map(function (interpolatedStyle) {
                            var _interpolatedStyle$st = interpolatedStyle.style,
                                colorR = _interpolatedStyle$st.colorR,
                                colorG = _interpolatedStyle$st.colorG,
                                colorB = _interpolatedStyle$st.colorB;

                            interpolatedStyle.style.color = 'rgb(' + Math.round(colorR) + ',' + Math.round(colorG) + ',' + Math.round(colorB) + ')';

                            return interpolatedStyle;
                        }), { showTooltip: showTooltip, hideTooltip: hideTooltip }));
                    }
                ));
            }
        );
    };

    return BubblePlaceholders;
}(_react.Component);

BubblePlaceholders.propTypes = _lodash2.default.omit(_BubbleProps.bubblePropTypes, ignoreProps);
BubblePlaceholders.defaultProps = _lodash2.default.omit(_BubbleProps.bubbleDefaultProps, ignoreProps);
exports.default = BubblePlaceholders;