'use strict';

exports.__esModule = true;
exports.calendarDefaultProps = exports.calendarPropTypes = undefined;

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

var _Nivo = require('../../../Nivo');

var _Nivo2 = _interopRequireDefault(_Nivo);

var _props = require('../../../props');

var _directions = require('../../../constants/directions');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/*
 * This file is part of the nivo project.
 *
 * Copyright 2016-present, Raphaël Benitte.
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */
var number = _propTypes2.default.number,
    string = _propTypes2.default.string,
    bool = _propTypes2.default.bool,
    func = _propTypes2.default.func,
    shape = _propTypes2.default.shape,
    oneOf = _propTypes2.default.oneOf,
    oneOfType = _propTypes2.default.oneOfType,
    instanceOf = _propTypes2.default.instanceOf,
    arrayOf = _propTypes2.default.arrayOf;

/**
 * Calendar components propTypes.
 *
 * @type {object}
 */

var calendarPropTypes = exports.calendarPropTypes = {
    width: number.isRequired,
    height: number.isRequired,
    margin: _props.marginPropType,
    from: oneOfType([string, instanceOf(Date)]).isRequired,
    to: oneOfType([string, instanceOf(Date)]).isRequired,
    data: arrayOf(shape({
        day: string.isRequired,
        value: number.isRequired
    })).isRequired,
    onDayClick: func.isRequired,
    direction: oneOf([_directions.DIRECTION_HORIZONTAL, _directions.DIRECTION_VERTICAL]),
    colorScale: _props.scalePropType.isRequired,
    emptyColor: string.isRequired,
    // years
    yearSpacing: number.isRequired,
    yearLegendOffset: number.isRequired,
    // days
    daySpacing: number.isRequired,
    dayBorderWidth: number.isRequired,
    dayBorderColor: string.isRequired,
    // months
    monthBorderWidth: number.isRequired,
    monthBorderColor: string.isRequired,
    monthLegendOffset: number.isRequired,
    // transitions
    animate: bool.isRequired,
    motionStiffness: number.isRequired, // react-motion
    motionDamping: number.isRequired, // react-motion
    transitionDuration: number.isRequired, // d3 transitions
    transitionEasing: string.isRequired, // d3 transitions
    transitionStaggering: number.isRequired // d3 transitions


    /**
     * Calendar components defaultProps.
     *
     * @type {object}
     */
};var calendarDefaultProps = exports.calendarDefaultProps = {
    margin: _Nivo2.default.defaults.margin,
    direction: _directions.DIRECTION_HORIZONTAL,
    onDayClick: function onDayClick() {},
    emptyColor: '#fff',
    // years
    yearSpacing: 30,
    yearLegendOffset: 10,
    // days
    daySpacing: 0,
    dayBorderWidth: 1,
    dayBorderColor: '#000',
    // months
    monthBorderWidth: 2,
    monthBorderColor: '#000',
    monthLegendOffset: 6,
    // transitions
    animate: false,
    motionStiffness: _Nivo2.default.defaults.motionStiffness, // react-motion
    motionDamping: _Nivo2.default.defaults.motionDamping, // react-motion
    transitionDuration: _Nivo2.default.defaults.transitionDuration, // d3 transitions
    transitionEasing: _Nivo2.default.defaults.transitionEasing, // d3 transitions
    transitionStaggering: 5 // d3 transitions
};