'use strict';

exports.__esModule = true;
exports.ChordDefaultProps = undefined;

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

var _compose = require('recompose/compose');

var _compose2 = _interopRequireDefault(_compose);

var _defaultProps = require('recompose/defaultProps');

var _defaultProps2 = _interopRequireDefault(_defaultProps);

var _pure = require('recompose/pure');

var _pure2 = _interopRequireDefault(_pure);

var _d3Chord = require('d3-chord');

var _d3Shape = require('d3-shape');

var _d3Color = require('d3-color');

var _hocs = require('../../../hocs');

var _colorUtils = require('../../../lib/colorUtils');

var _Container = require('../Container');

var _Container2 = _interopRequireDefault(_Container);

var _SvgWrapper = require('../SvgWrapper');

var _SvgWrapper2 = _interopRequireDefault(_SvgWrapper);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/*
 * This file is part of the nivo project.
 *
 * Copyright 2016-present, Raphaël Benitte.
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */
var Chord = function Chord(_ref) {
    var data = _ref.data,
        margin = _ref.margin,
        width = _ref.width,
        height = _ref.height,
        outerWidth = _ref.outerWidth,
        outerHeight = _ref.outerHeight,
        padAngle = _ref.padAngle,
        innerRadiusRatio = _ref.innerRadiusRatio,
        innerRadiusOffset = _ref.innerRadiusOffset,
        ribbonOpacity = _ref.ribbonOpacity,
        ribbonBorderWidth = _ref.ribbonBorderWidth,
        arcOpacity = _ref.arcOpacity,
        arcBorderWidth = _ref.arcBorderWidth,
        colors = _ref.colors,
        isInteractive = _ref.isInteractive;

    var centerX = width / 2;
    var centerY = height / 2;

    var color = (0, _colorUtils.getColorRange)(colors);

    var radius = Math.min(width, height) / 2;
    var arcInnerRadius = radius * innerRadiusRatio;
    var ribbonRadius = radius * (innerRadiusRatio - innerRadiusOffset);

    var chord = (0, _d3Chord.chord)().padAngle(padAngle);

    var arc = (0, _d3Shape.arc)().innerRadius(arcInnerRadius).outerRadius(radius);

    var ribbon = (0, _d3Chord.ribbon)().radius(ribbonRadius);

    var ribbons = chord(data);
    var arcs = ribbons.groups;

    return _react2.default.createElement(
        _Container2.default,
        { isInteractive: isInteractive },
        function (_ref2) {
            var showTooltip = _ref2.showTooltip,
                hideTooltip = _ref2.hideTooltip;

            return _react2.default.createElement(
                _SvgWrapper2.default,
                { width: outerWidth, height: outerHeight, margin: margin },
                _react2.default.createElement(
                    'g',
                    { transform: 'translate(' + centerX + ', ' + centerY + ')' },
                    _react2.default.createElement(
                        'g',
                        null,
                        ribbons.map(function (d) {
                            var c = (0, _d3Color.rgb)(color(d.source.index));
                            c = (0, _d3Color.rgb)(c.r, c.g, c.b, ribbonOpacity);

                            return _react2.default.createElement('path', {
                                key: 'ribbon.' + d.source.index + '.' + d.target.index,
                                className: 'nivo_chord_ribbon',
                                d: ribbon(d),
                                fill: c,
                                stroke: c,
                                strokeWidth: ribbonBorderWidth
                            });
                        })
                    ),
                    _react2.default.createElement(
                        'g',
                        null,
                        arcs.map(function (d) {
                            var c = (0, _d3Color.rgb)(color(d.index));
                            c = (0, _d3Color.rgb)(c.r, c.g, c.b, arcOpacity);

                            return _react2.default.createElement('path', {
                                key: 'arc.' + d.index,
                                className: 'nivo_chord_arc',
                                d: arc(d),
                                fill: c,
                                stroke: c,
                                strokeWidth: arcBorderWidth
                            });
                        })
                    )
                )
            );
        }
    );
};

Chord.propTypes = {
    data: _propTypes2.default.array.isRequired,

    padAngle: _propTypes2.default.number.isRequired,
    innerRadiusRatio: _propTypes2.default.number.isRequired,
    innerRadiusOffset: _propTypes2.default.number.isRequired,

    ribbonOpacity: _propTypes2.default.number.isRequired,
    ribbonBorderWidth: _propTypes2.default.number.isRequired,

    // colors
    colors: _propTypes2.default.any.isRequired,

    // interactivity
    isInteractive: _propTypes2.default.bool.isRequired
};

var ChordDefaultProps = exports.ChordDefaultProps = {
    padAngle: 0,
    innerRadiusRatio: 0.9,
    innerRadiusOffset: 0,

    ribbonOpacity: 0.5,
    ribbonBorderWidth: 1,

    arcOpacity: 1,
    arcBorderWidth: 1,

    // colors
    colors: 'nivo',

    // interactivity
    isInteractive: true
};

var enhance = (0, _compose2.default)((0, _defaultProps2.default)(ChordDefaultProps), (0, _hocs.withDimensions)(), _pure2.default);

exports.default = enhance(Chord);