'use strict';

exports.__esModule = true;
exports.LineDefaultProps = undefined;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; /*
                                                                                                                                                                                                                                                                   * This file is part of the nivo project.
                                                                                                                                                                                                                                                                   *
                                                                                                                                                                                                                                                                   * Copyright 2016-present, Raphaël Benitte.
                                                                                                                                                                                                                                                                   *
                                                                                                                                                                                                                                                                   * For the full copyright and license information, please view the LICENSE
                                                                                                                                                                                                                                                                   * file that was distributed with this source code.
                                                                                                                                                                                                                                                                   */


var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

var _lodash = require('lodash');

var _d3Shape = require('d3-shape');

var _compose = require('recompose/compose');

var _compose2 = _interopRequireDefault(_compose);

var _pure = require('recompose/pure');

var _pure2 = _interopRequireDefault(_pure);

var _withPropsOnChange = require('recompose/withPropsOnChange');

var _withPropsOnChange2 = _interopRequireDefault(_withPropsOnChange);

var _defaultProps = require('recompose/defaultProps');

var _defaultProps2 = _interopRequireDefault(_defaultProps);

var _props = require('../../../props');

var _colorUtils = require('../../../lib/colorUtils');

var _hocs = require('../../../hocs');

var _Container = require('../Container');

var _Container2 = _interopRequireDefault(_Container);

var _SvgWrapper = require('../SvgWrapper');

var _SvgWrapper2 = _interopRequireDefault(_SvgWrapper);

var _line = require('../../../lib/charts/line');

var _Axes = require('../../axes/Axes');

var _Axes2 = _interopRequireDefault(_Axes);

var _Grid = require('../../axes/Grid');

var _Grid2 = _interopRequireDefault(_Grid);

var _LineLines = require('./LineLines');

var _LineLines2 = _interopRequireDefault(_LineLines);

var _LineSlices = require('./LineSlices');

var _LineSlices2 = _interopRequireDefault(_LineSlices);

var _LineMarkers = require('./LineMarkers');

var _LineMarkers2 = _interopRequireDefault(_LineMarkers);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var Line = function Line(_ref) {
    var lines = _ref.lines,
        lineGenerator = _ref.lineGenerator,
        xScale = _ref.xScale,
        yScale = _ref.yScale,
        slices = _ref.slices,
        margin = _ref.margin,
        width = _ref.width,
        height = _ref.height,
        outerWidth = _ref.outerWidth,
        outerHeight = _ref.outerHeight,
        axisTop = _ref.axisTop,
        axisRight = _ref.axisRight,
        axisBottom = _ref.axisBottom,
        axisLeft = _ref.axisLeft,
        enableGridX = _ref.enableGridX,
        enableGridY = _ref.enableGridY,
        enableMarkers = _ref.enableMarkers,
        markersSize = _ref.markersSize,
        markersColor = _ref.markersColor,
        markersBorderWidth = _ref.markersBorderWidth,
        markersBorderColor = _ref.markersBorderColor,
        enableMarkersLabel = _ref.enableMarkersLabel,
        markersLabel = _ref.markersLabel,
        markersLabelFormat = _ref.markersLabelFormat,
        markersLabelYOffset = _ref.markersLabelYOffset,
        theme = _ref.theme,
        animate = _ref.animate,
        motionStiffness = _ref.motionStiffness,
        motionDamping = _ref.motionDamping,
        isInteractive = _ref.isInteractive,
        enableStackTooltip = _ref.enableStackTooltip;

    var motionProps = {
        animate: animate,
        motionDamping: motionDamping,
        motionStiffness: motionStiffness
    };

    return _react2.default.createElement(
        _Container2.default,
        { isInteractive: isInteractive },
        function (_ref2) {
            var showTooltip = _ref2.showTooltip,
                hideTooltip = _ref2.hideTooltip;
            return _react2.default.createElement(
                _SvgWrapper2.default,
                { width: outerWidth, height: outerHeight, margin: margin },
                _react2.default.createElement(_Grid2.default, _extends({
                    theme: theme,
                    width: width,
                    height: height,
                    xScale: enableGridX ? xScale : null,
                    yScale: enableGridY ? yScale : null
                }, motionProps)),
                _react2.default.createElement(_Axes2.default, _extends({
                    xScale: xScale,
                    yScale: yScale,
                    width: width,
                    height: height,
                    theme: theme,
                    top: axisTop,
                    right: axisRight,
                    bottom: axisBottom,
                    left: axisLeft
                }, motionProps)),
                _react2.default.createElement(_LineLines2.default, _extends({ lines: lines, lineGenerator: lineGenerator }, motionProps)),
                isInteractive && enableStackTooltip && _react2.default.createElement(_LineSlices2.default, {
                    slices: slices,
                    height: height,
                    showTooltip: showTooltip,
                    hideTooltip: hideTooltip
                }),
                enableMarkers && _react2.default.createElement(_LineMarkers2.default, _extends({
                    lines: lines,
                    size: markersSize,
                    color: (0, _colorUtils.getInheritedColorGenerator)(markersColor),
                    borderWidth: markersBorderWidth,
                    borderColor: (0, _colorUtils.getInheritedColorGenerator)(markersBorderColor),
                    enableLabel: enableMarkersLabel,
                    label: markersLabel,
                    labelFormat: markersLabelFormat,
                    labelYOffset: markersLabelYOffset,
                    theme: theme
                }, motionProps))
            );
        }
    );
};

Line.propTypes = {
    // data
    data: _propTypes2.default.arrayOf(_propTypes2.default.shape({
        id: _propTypes2.default.string.isRequired,
        data: _propTypes2.default.arrayOf(_propTypes2.default.shape({
            x: _propTypes2.default.oneOfType([_propTypes2.default.number, _propTypes2.default.string]).isRequired,
            y: _propTypes2.default.oneOfType([_propTypes2.default.number, _propTypes2.default.string]).isRequired
        })).isRequired
    })).isRequired,

    stacked: _propTypes2.default.bool.isRequired,
    curve: _props.lineCurvePropType.isRequired,
    lineGenerator: _propTypes2.default.func.isRequired,

    lines: _propTypes2.default.array.isRequired,
    slices: _propTypes2.default.array.isRequired,

    xScale: _propTypes2.default.func.isRequired,
    yScale: _propTypes2.default.func.isRequired,

    // axes & grid
    axisTop: _propTypes2.default.object,
    axisRight: _propTypes2.default.object,
    axisBottom: _propTypes2.default.object,
    axisLeft: _propTypes2.default.object,
    enableGridX: _propTypes2.default.bool.isRequired,
    enableGridY: _propTypes2.default.bool.isRequired,

    // markers
    enableMarkers: _propTypes2.default.bool.isRequired,
    markersSize: _propTypes2.default.number.isRequired,
    markersColor: _propTypes2.default.any.isRequired,
    markersBorderWidth: _propTypes2.default.number.isRequired,
    markersBorderColor: _propTypes2.default.any.isRequired,
    enableMarkersLabel: _propTypes2.default.bool.isRequired,

    // theming
    getColor: _propTypes2.default.func.isRequired,

    // interactivity
    isInteractive: _propTypes2.default.bool.isRequired,

    // stack tooltip
    enableStackTooltip: _propTypes2.default.bool.isRequired
};

var LineDefaultProps = exports.LineDefaultProps = {
    stacked: false,
    curve: 'linear',

    // axes & grid
    axisBottom: {},
    axisLeft: {},
    enableGridX: true,
    enableGridY: true,

    // markers
    enableMarkers: true,
    markersSize: 6,
    markersColor: 'inherit',
    markersBorderWidth: 0,
    markersBorderColor: 'inherit',
    enableMarkersLabel: false,

    // theming
    colors: 'nivo',
    colorBy: 'id',

    // interactivity
    isInteractive: true,

    // stack tooltip
    enableStackTooltip: true
};

var enhance = (0, _compose2.default)((0, _defaultProps2.default)(LineDefaultProps), (0, _hocs.withTheme)(), (0, _hocs.withColors)(), (0, _hocs.withDimensions)(), (0, _hocs.withMotion)(), (0, _withPropsOnChange2.default)(['curve'], function (_ref3) {
    var curve = _ref3.curve;
    return {
        lineGenerator: (0, _d3Shape.line)().x(function (d) {
            return d.x;
        }).y(function (d) {
            return d.y;
        }).curve((0, _props.curveFromProp)(curve))
    };
}), (0, _withPropsOnChange2.default)(function (props, nextProps) {
    return props.data !== nextProps.data || props.stacked !== nextProps.stacked || props.width !== nextProps.width || props.height !== nextProps.height;
}, function (_ref4) {
    var data = _ref4.data,
        stacked = _ref4.stacked,
        width = _ref4.width,
        height = _ref4.height,
        margin = _ref4.margin;

    var scales = void 0;
    if (stacked === true) {
        scales = (0, _line.getStackedScales)(data, width, height);
    } else {
        scales = (0, _line.getScales)(data, width, height);
    }

    return _extends({
        margin: margin,
        width: width,
        height: height
    }, scales);
}), (0, _withPropsOnChange2.default)(['getColor', 'xScale', 'yScale'], function (_ref5) {
    var data = _ref5.data,
        stacked = _ref5.stacked,
        xScale = _ref5.xScale,
        yScale = _ref5.yScale,
        getColor = _ref5.getColor;

    var lines = void 0;
    if (stacked === true) {
        lines = (0, _line.generateStackedLines)(data, xScale, yScale, getColor);
    } else {
        lines = (0, _line.generateLines)(data, xScale, yScale, getColor);
    }

    var slices = xScale.domain().map(function (id, i) {
        var points = (0, _lodash.sortBy)(lines.map(function (line) {
            return {
                id: line.id,
                value: line.points[i].value,
                y: line.points[i].y,
                color: line.color
            };
        }), 'y');

        return {
            id: id,
            x: xScale(id),
            points: points
        };
    });

    return { lines: lines, slices: slices };
}), _pure2.default);

exports.default = enhance(Line);