'use strict';

exports.__esModule = true;
exports.RadarDefaultProps = undefined;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; /*
                                                                                                                                                                                                                                                                   * This file is part of the nivo project.
                                                                                                                                                                                                                                                                   *
                                                                                                                                                                                                                                                                   * Copyright 2016-present, Raphaël Benitte.
                                                                                                                                                                                                                                                                   *
                                                                                                                                                                                                                                                                   * For the full copyright and license information, please view the LICENSE
                                                                                                                                                                                                                                                                   * file that was distributed with this source code.
                                                                                                                                                                                                                                                                   */


var _lodash = require('lodash');

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

var _props = require('../../../props');

var _hocs = require('../../../hocs');

var _SvgWrapper = require('../SvgWrapper');

var _SvgWrapper2 = _interopRequireDefault(_SvgWrapper);

var _d3Scale = require('d3-scale');

var _RadarShapes = require('./RadarShapes');

var _RadarShapes2 = _interopRequireDefault(_RadarShapes);

var _RadarGrid = require('./RadarGrid');

var _RadarGrid2 = _interopRequireDefault(_RadarGrid);

var _RadarMarkers = require('./RadarMarkers');

var _RadarMarkers2 = _interopRequireDefault(_RadarMarkers);

var _compose = require('recompose/compose');

var _compose2 = _interopRequireDefault(_compose);

var _pure = require('recompose/pure');

var _pure2 = _interopRequireDefault(_pure);

var _withPropsOnChange = require('recompose/withPropsOnChange');

var _withPropsOnChange2 = _interopRequireDefault(_withPropsOnChange);

var _defaultProps = require('recompose/defaultProps');

var _defaultProps2 = _interopRequireDefault(_defaultProps);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var Radar = function Radar(_ref) {
    var facets = _ref.facets,
        data = _ref.data,
        curveInterpolator = _ref.curveInterpolator,
        radius = _ref.radius,
        radiusScale = _ref.radiusScale,
        angleStep = _ref.angleStep,
        centerX = _ref.centerX,
        centerY = _ref.centerY,
        margin = _ref.margin,
        outerWidth = _ref.outerWidth,
        outerHeight = _ref.outerHeight,
        borderWidth = _ref.borderWidth,
        borderColor = _ref.borderColor,
        gridLevels = _ref.gridLevels,
        gridShape = _ref.gridShape,
        gridLabelOffset = _ref.gridLabelOffset,
        enableMarkers = _ref.enableMarkers,
        markersSize = _ref.markersSize,
        markersColor = _ref.markersColor,
        markersBorderWidth = _ref.markersBorderWidth,
        markersBorderColor = _ref.markersBorderColor,
        enableMarkersLabel = _ref.enableMarkersLabel,
        markersLabel = _ref.markersLabel,
        markersLabelFormat = _ref.markersLabelFormat,
        markersLabelYOffset = _ref.markersLabelYOffset,
        theme = _ref.theme,
        fillOpacity = _ref.fillOpacity,
        animate = _ref.animate,
        motionStiffness = _ref.motionStiffness,
        motionDamping = _ref.motionDamping,
        isInteractive = _ref.isInteractive;

    var motionProps = {
        animate: animate,
        motionDamping: motionDamping,
        motionStiffness: motionStiffness
    };

    return _react2.default.createElement(
        _SvgWrapper2.default,
        { width: outerWidth, height: outerHeight, margin: margin },
        _react2.default.createElement(
            'g',
            { transform: 'translate(' + centerX + ', ' + centerY + ')' },
            _react2.default.createElement(_RadarGrid2.default, _extends({
                levels: gridLevels,
                shape: gridShape,
                radius: radius,
                angleStep: angleStep,
                theme: theme,
                facets: facets,
                labelOffset: gridLabelOffset
            }, motionProps)),
            _react2.default.createElement(_RadarShapes2.default, _extends({
                data: data,
                radiusScale: radiusScale,
                angleStep: angleStep,
                curveInterpolator: curveInterpolator,
                borderWidth: borderWidth,
                borderColor: borderColor,
                fillOpacity: fillOpacity
            }, motionProps)),
            enableMarkers && _react2.default.createElement(_RadarMarkers2.default, _extends({
                facets: facets,
                data: data,
                radiusScale: radiusScale,
                angleStep: angleStep,
                size: markersSize,
                color: markersColor,
                borderWidth: markersBorderWidth,
                borderColor: markersBorderColor,
                enableLabel: enableMarkersLabel,
                label: markersLabel,
                labelFormat: markersLabelFormat,
                labelYOffset: markersLabelYOffset,
                theme: theme
            }, motionProps))
        )
    );
};

Radar.propTypes = {
    // data
    facets: _propTypes2.default.arrayOf(_propTypes2.default.oneOfType([_propTypes2.default.string, _propTypes2.default.number])).isRequired,
    data: _propTypes2.default.arrayOf(_propTypes2.default.shape({
        id: _propTypes2.default.string.isRequired,
        data: _propTypes2.default.arrayOf(_propTypes2.default.number).isRequired
    })).isRequired,

    curve: _props.closedCurvePropType.isRequired,
    curveInterpolator: _propTypes2.default.func.isRequired,

    // border
    borderWidth: _propTypes2.default.number.isRequired,
    borderColor: _propTypes2.default.oneOfType([_propTypes2.default.string, _propTypes2.default.func]),

    // grid
    gridLevels: _propTypes2.default.number,
    gridShape: _propTypes2.default.oneOf(['circular', 'linear']),
    gridLabelOffset: _propTypes2.default.number,

    // markers
    enableMarkers: _propTypes2.default.bool.isRequired,
    markersSize: _propTypes2.default.number,
    markersColor: _propTypes2.default.any,
    markersBorderWidth: _propTypes2.default.number,
    markersBorderColor: _propTypes2.default.any,
    enableMarkersLabel: _propTypes2.default.bool,
    markersLabel: _propTypes2.default.oneOfType([_propTypes2.default.string, _propTypes2.default.func]),
    markersLabelFormat: _propTypes2.default.string,
    markersLabelYOffset: _propTypes2.default.number,

    // theming
    getColor: _propTypes2.default.func.isRequired,
    fillOpacity: _propTypes2.default.number.isRequired,

    isInteractive: _propTypes2.default.bool.isRequired
};

var RadarDefaultProps = exports.RadarDefaultProps = {
    curve: 'linearClosed',

    // border
    borderWidth: 2,
    borderColor: 'inherit',

    // grid
    gridLevels: 5,
    gridShape: 'circular',
    gridLabelOffset: 16,

    // markers
    enableMarkers: true,

    // theming
    fillOpacity: 0.15,

    // interactivity
    isInteractive: true
};

var enhance = (0, _compose2.default)((0, _defaultProps2.default)(RadarDefaultProps), (0, _hocs.withTheme)(), (0, _hocs.withColors)(), (0, _hocs.withCurve)(), (0, _hocs.withDimensions)(), (0, _hocs.withMotion)(), (0, _withPropsOnChange2.default)(function (props, nextProps) {
    return props.facets !== nextProps.facets || props.data !== nextProps.data || props.width !== nextProps.width || props.height !== nextProps.height || props.getColor !== nextProps.getColor;
}, function (_ref2) {
    var facets = _ref2.facets,
        data = _ref2.data,
        getColor = _ref2.getColor,
        width = _ref2.width,
        height = _ref2.height;

    var maxValue = (0, _lodash.max)(data.reduce(function (acc, serie) {
        return [].concat(acc, serie.data);
    }, []));

    var radius = Math.min(width, height) / 2;
    var radiusScale = (0, _d3Scale.scaleLinear)().range([0, radius]).domain([0, maxValue]);

    return {
        data: data.map(function (d) {
            return Object.assign({}, d, { color: getColor(d) });
        }),
        radius: radius,
        radiusScale: radiusScale,
        centerX: width / 2,
        centerY: height / 2,
        angleStep: Math.PI * 2 / facets.length
    };
}), _pure2.default);

exports.default = enhance(Radar);