'use strict';

exports.__esModule = true;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

var _reactMotion = require('react-motion');

var _props2 = require('../../../props');

var _colorUtils = require('../../../lib/colorUtils');

var _arcUtils = require('../../../lib/arcUtils');

var _propertiesConverters = require('../../../lib/propertiesConverters');

var _MarkersItem = require('../../markers/MarkersItem');

var _MarkersItem2 = _interopRequireDefault(_MarkersItem);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; } /*
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                * This file is part of the nivo project.
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                *
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                * Copyright 2016-present, Raphaël Benitte.
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                *
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                * For the full copyright and license information, please view the LICENSE
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                * file that was distributed with this source code.
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                */


var RadarMarkers = function (_Component) {
    _inherits(RadarMarkers, _Component);

    function RadarMarkers() {
        _classCallCheck(this, RadarMarkers);

        return _possibleConstructorReturn(this, _Component.apply(this, arguments));
    }

    RadarMarkers.prototype.render = function render() {
        var _props = this.props,
            facets = _props.facets,
            data = _props.data,
            radiusScale = _props.radiusScale,
            angleStep = _props.angleStep,
            size = _props.size,
            color = _props.color,
            borderWidth = _props.borderWidth,
            borderColor = _props.borderColor,
            enableLabel = _props.enableLabel,
            label = _props.label,
            labelFormat = _props.labelFormat,
            labelYOffset = _props.labelYOffset,
            theme = _props.theme,
            animate = _props.animate,
            motionStiffness = _props.motionStiffness,
            motionDamping = _props.motionDamping;


        var fillColor = (0, _colorUtils.getInheritedColorGenerator)(color);
        var strokeColor = (0, _colorUtils.getInheritedColorGenerator)(borderColor);
        var getLabel = (0, _propertiesConverters.getLabelGenerator)(label, labelFormat);

        var points = data.reduce(function (acc, serie) {
            var id = serie.id,
                serieData = serie.data;


            return [].concat(acc, serieData.map(function (d, i) {
                var pointData = { value: d, serie: serie, facet: facets[i] };

                return _extends({
                    key: id + '.' + i,
                    fill: fillColor(serie),
                    stroke: strokeColor(serie)
                }, (0, _arcUtils.positionFromAngle)(angleStep * i - Math.PI / 2, radiusScale(d)), pointData, {
                    label: enableLabel ? getLabel(pointData) : null
                });
            }));
        }, []);

        if (animate !== true) {
            return _react2.default.createElement(
                'g',
                null,
                points.map(function (point) {
                    return _react2.default.createElement(_MarkersItem2.default, {
                        key: point.key,
                        x: point.x,
                        y: point.y,
                        size: size,
                        color: point.fill,
                        borderWidth: borderWidth,
                        borderColor: point.stroke,
                        label: point.label,
                        labelYOffset: labelYOffset,
                        theme: theme
                    });
                })
            );
        }

        var springConfig = {
            motionDamping: motionDamping,
            motionStiffness: motionStiffness
        };

        return _react2.default.createElement(
            _reactMotion.TransitionMotion,
            {
                styles: points.map(function (point) {
                    return {
                        key: point.key,
                        data: point,
                        style: {
                            x: (0, _reactMotion.spring)(point.x, springConfig),
                            y: (0, _reactMotion.spring)(point.y, springConfig),
                            size: (0, _reactMotion.spring)(size, springConfig)
                        }
                    };
                })
            },
            function (interpolatedStyles) {
                return _react2.default.createElement(
                    'g',
                    null,
                    interpolatedStyles.map(function (_ref) {
                        var key = _ref.key,
                            style = _ref.style,
                            point = _ref.data;
                        return _react2.default.createElement(_MarkersItem2.default, _extends({
                            key: key
                        }, style, {
                            color: point.fill,
                            borderWidth: borderWidth,
                            borderColor: point.stroke,
                            label: point.label,
                            labelYOffset: labelYOffset,
                            theme: theme
                        }));
                    })
                );
            }
        );
    };

    return RadarMarkers;
}(_react.Component);

RadarMarkers.propTypes = _extends({
    facets: _propTypes2.default.arrayOf(_propTypes2.default.oneOfType([_propTypes2.default.string, _propTypes2.default.number])).isRequired,
    data: _propTypes2.default.arrayOf(_propTypes2.default.shape({
        id: _propTypes2.default.string.isRequired,
        color: _propTypes2.default.string.isRequired,
        data: _propTypes2.default.arrayOf(_propTypes2.default.number).isRequired
    })),
    radiusScale: _propTypes2.default.func.isRequired,
    angleStep: _propTypes2.default.number.isRequired,
    size: _propTypes2.default.number.isRequired,
    color: _propTypes2.default.oneOfType([_propTypes2.default.string, _propTypes2.default.func]).isRequired,
    borderWidth: _propTypes2.default.number.isRequired,
    borderColor: _propTypes2.default.oneOfType([_propTypes2.default.string, _propTypes2.default.func]).isRequired,

    // labels
    enableLabel: _propTypes2.default.bool.isRequired,
    label: _propTypes2.default.oneOfType([_propTypes2.default.string, _propTypes2.default.func]).isRequired,
    labelFormat: _propTypes2.default.string,
    labelYOffset: _propTypes2.default.number,

    // theming
    theme: _propTypes2.default.shape({
        markers: _propTypes2.default.shape({
            textColor: _propTypes2.default.string.isRequired,
            fontSize: _propTypes2.default.string.isRequired
        }).isRequired
    }).isRequired

}, _props2.motionPropTypes);
RadarMarkers.defaultProps = {
    size: 6,
    color: 'inherit',
    borderWidth: 0,
    borderColor: 'inherit',

    // labels
    enableLabel: false,
    label: 'value'
};
exports.default = RadarMarkers;