'use strict';

exports.__esModule = true;
exports.StreamDefaultProps = undefined;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; /*
                                                                                                                                                                                                                                                                   * This file is part of the nivo project.
                                                                                                                                                                                                                                                                   *
                                                                                                                                                                                                                                                                   * Copyright 2016-present, Raphaël Benitte.
                                                                                                                                                                                                                                                                   *
                                                                                                                                                                                                                                                                   * For the full copyright and license information, please view the LICENSE
                                                                                                                                                                                                                                                                   * file that was distributed with this source code.
                                                                                                                                                                                                                                                                   */


var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

var _lodash = require('lodash');

var _d3Shape = require('d3-shape');

var _d3Scale = require('d3-scale');

var _compose = require('recompose/compose');

var _compose2 = _interopRequireDefault(_compose);

var _pure = require('recompose/pure');

var _pure2 = _interopRequireDefault(_pure);

var _withPropsOnChange = require('recompose/withPropsOnChange');

var _withPropsOnChange2 = _interopRequireDefault(_withPropsOnChange);

var _defaultProps = require('recompose/defaultProps');

var _defaultProps2 = _interopRequireDefault(_defaultProps);

var _props = require('../../../props');

var _hocs = require('../../../hocs');

var _colorUtils = require('../../../lib/colorUtils');

var _SvgWrapper = require('../SvgWrapper');

var _SvgWrapper2 = _interopRequireDefault(_SvgWrapper);

var _Container = require('../Container');

var _Container2 = _interopRequireDefault(_Container);

var _Axes = require('../../axes/Axes');

var _Axes2 = _interopRequireDefault(_Axes);

var _Grid = require('../../axes/Grid');

var _Grid2 = _interopRequireDefault(_Grid);

var _StreamLayers = require('./StreamLayers');

var _StreamLayers2 = _interopRequireDefault(_StreamLayers);

var _StreamSlices = require('./StreamSlices');

var _StreamSlices2 = _interopRequireDefault(_StreamSlices);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var stackMin = function stackMin(layers) {
    return (0, _lodash.min)(layers.reduce(function (acc, layer) {
        return [].concat(acc, layer.map(function (d) {
            return d[0];
        }));
    }, []));
};
var stackMax = function stackMax(layers) {
    return (0, _lodash.max)(layers.reduce(function (acc, layer) {
        return [].concat(acc, layer.map(function (d) {
            return d[1];
        }));
    }, []));
};

var Stream = function Stream(_ref) {
    var data = _ref.data,
        keys = _ref.keys,
        xScale = _ref.xScale,
        yScale = _ref.yScale,
        layers = _ref.layers,
        areaGenerator = _ref.areaGenerator,
        margin = _ref.margin,
        width = _ref.width,
        height = _ref.height,
        outerWidth = _ref.outerWidth,
        outerHeight = _ref.outerHeight,
        axisTop = _ref.axisTop,
        axisRight = _ref.axisRight,
        axisBottom = _ref.axisBottom,
        axisLeft = _ref.axisLeft,
        enableGridX = _ref.enableGridX,
        enableGridY = _ref.enableGridY,
        theme = _ref.theme,
        getColor = _ref.getColor,
        fillOpacity = _ref.fillOpacity,
        animate = _ref.animate,
        motionStiffness = _ref.motionStiffness,
        motionDamping = _ref.motionDamping,
        isInteractive = _ref.isInteractive,
        enableStackTooltip = _ref.enableStackTooltip;

    var enhancedLayers = layers.map(function (points, i) {
        var layer = points.map(function (_ref2, i) {
            var y1 = _ref2[0],
                y2 = _ref2[1];
            return {
                index: i,
                value: y2 - y1,
                x: xScale(i),
                y1: yScale(y1),
                y2: yScale(y2)
            };
        });

        return {
            id: keys[i],
            layer: layer,
            path: areaGenerator(layer),
            color: getColor(i)
        };
    });

    var slices = (0, _lodash.range)(data.length).map(function (i) {
        return {
            index: i,
            x: enhancedLayers[0].layer[i].x,
            stack: (0, _lodash.sortBy)(enhancedLayers.map(function (layer) {
                return _extends({
                    id: layer.id,
                    color: layer.color
                }, layer.layer[i]);
            }), 'y2')
        };
    });

    var motionProps = {
        animate: animate,
        motionDamping: motionDamping,
        motionStiffness: motionStiffness
    };

    return _react2.default.createElement(
        _Container2.default,
        { isInteractive: isInteractive },
        function (_ref3) {
            var showTooltip = _ref3.showTooltip,
                hideTooltip = _ref3.hideTooltip;
            return _react2.default.createElement(
                _SvgWrapper2.default,
                { width: outerWidth, height: outerHeight, margin: margin },
                _react2.default.createElement(_Grid2.default, _extends({
                    theme: theme,
                    width: width,
                    height: height,
                    xScale: enableGridX ? xScale : null,
                    yScale: enableGridY ? yScale : null
                }, motionProps)),
                _react2.default.createElement(_StreamLayers2.default, _extends({
                    layers: enhancedLayers,
                    fillOpacity: fillOpacity,
                    showTooltip: showTooltip,
                    hideTooltip: hideTooltip
                }, motionProps)),
                _react2.default.createElement(_Axes2.default, _extends({
                    xScale: xScale,
                    yScale: yScale,
                    width: width,
                    height: height,
                    theme: theme,
                    top: axisTop,
                    right: axisRight,
                    bottom: axisBottom,
                    left: axisLeft
                }, motionProps)),
                isInteractive && enableStackTooltip && _react2.default.createElement(_StreamSlices2.default, {
                    slices: slices,
                    height: height,
                    showTooltip: showTooltip,
                    hideTooltip: hideTooltip
                })
            );
        }
    );
};

Stream.propTypes = {
    // data
    data: _propTypes2.default.arrayOf(_propTypes2.default.object).isRequired,
    keys: _propTypes2.default.array.isRequired,

    stack: _propTypes2.default.func.isRequired,
    xScale: _propTypes2.default.func.isRequired,
    yScale: _propTypes2.default.func.isRequired,

    order: _props.stackOrderPropType.isRequired,
    offsetType: _props.stackOffsetPropType.isRequired,
    curve: _props.areaCurvePropType.isRequired,
    areaGenerator: _propTypes2.default.func.isRequired,

    // axes & grid
    axisTop: _propTypes2.default.object,
    axisRight: _propTypes2.default.object,
    axisBottom: _propTypes2.default.object,
    axisLeft: _propTypes2.default.object,
    enableGridX: _propTypes2.default.bool.isRequired,
    enableGridY: _propTypes2.default.bool.isRequired,

    // theming
    colors: _propTypes2.default.any.isRequired,
    fillOpacity: _propTypes2.default.number.isRequired,
    getColor: _propTypes2.default.func.isRequired,

    // interactivity
    isInteractive: _propTypes2.default.bool,

    // stack tooltip
    enableStackTooltip: _propTypes2.default.bool.isRequired
};

var StreamDefaultProps = exports.StreamDefaultProps = {
    order: 'none',
    offsetType: 'wiggle',
    curve: 'catmullRom',

    // axes & grid
    axisBottom: {},
    enableGridX: true,
    enableGridY: false,

    // theming
    colors: 'nivo',
    fillOpacity: 1,

    // interactivity
    isInteractive: true,

    // stack tooltip
    enableStackTooltip: true
};

var enhance = (0, _compose2.default)((0, _defaultProps2.default)(StreamDefaultProps), (0, _hocs.withTheme)(), (0, _hocs.withCurve)(), (0, _hocs.withDimensions)(), (0, _hocs.withMotion)(), (0, _withPropsOnChange2.default)(['curveInterpolator'], function (_ref4) {
    var curveInterpolator = _ref4.curveInterpolator;
    return {
        areaGenerator: (0, _d3Shape.area)().x(function (_ref5) {
            var x = _ref5.x;
            return x;
        }).y0(function (_ref6) {
            var y1 = _ref6.y1;
            return y1;
        }).y1(function (_ref7) {
            var y2 = _ref7.y2;
            return y2;
        }).curve(curveInterpolator)
    };
}), (0, _withPropsOnChange2.default)(['colors'], function (_ref8) {
    var colors = _ref8.colors;
    return {
        getColor: (0, _colorUtils.getColorRange)(colors)
    };
}), (0, _withPropsOnChange2.default)(['keys', 'offsetType', 'order'], function (_ref9) {
    var keys = _ref9.keys,
        offsetType = _ref9.offsetType,
        order = _ref9.order;
    return {
        stack: (0, _d3Shape.stack)().keys(keys).offset((0, _props.stackOffsetFromProp)(offsetType)).order((0, _props.stackOrderFromProp)(order))
    };
}), (0, _withPropsOnChange2.default)(['stack', 'data', 'width', 'height'], function (_ref10) {
    var stack = _ref10.stack,
        data = _ref10.data,
        width = _ref10.width,
        height = _ref10.height;

    var layers = stack(data);

    var minValue = stackMin(layers);
    var maxValue = stackMax(layers);

    return {
        layers: layers,
        xScale: (0, _d3Scale.scalePoint)().domain((0, _lodash.range)(data.length)).range([0, width]),
        yScale: (0, _d3Scale.scaleLinear)().domain([minValue, maxValue]).range([height, 0])
    };
}), _pure2.default);

exports.default = enhance(Stream);