'use strict';

exports.__esModule = true;
exports.computeTreeMap = exports.tilingMethods = undefined;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; /*
                                                                                                                                                                                                                                                                   * This file is part of the nivo project.
                                                                                                                                                                                                                                                                   *
                                                                                                                                                                                                                                                                   * Copyright 2016-present, Raphaël Benitte.
                                                                                                                                                                                                                                                                   *
                                                                                                                                                                                                                                                                   * For the full copyright and license information, please view the LICENSE
                                                                                                                                                                                                                                                                   * file that was distributed with this source code.
                                                                                                                                                                                                                                                                   */

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _d3Hierarchy = require('d3-hierarchy');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var tilingMethods = exports.tilingMethods = {
    binary: _d3Hierarchy.treemapBinary,
    dice: _d3Hierarchy.treemapDice,
    slice: _d3Hierarchy.treemapSlice,
    sliceDice: _d3Hierarchy.treemapSliceDice,
    squarify: _d3Hierarchy.treemapSquarify,
    resquarify: _d3Hierarchy.treemapResquarify

    /**
     * @param {number}   width
     * @param {number}   height
     * @param {object}   _root
     * @param {boolean}  leavesOnly
     * @param {string}   tile
     * @param {number}   innerPadding
     * @param {number}   outerPadding
     * @param {function} identity
     * @param {function} value
     * @param {function} color
     */
};var computeTreeMap = exports.computeTreeMap = function computeTreeMap(_ref) {
    var width = _ref.width,
        height = _ref.height,
        _root = _ref.root,
        leavesOnly = _ref.leavesOnly,
        tile = _ref.tile,
        innerPadding = _ref.innerPadding,
        outerPadding = _ref.outerPadding,
        identity = _ref.identity,
        value = _ref.value,
        color = _ref.color;

    var treemap = (0, _d3Hierarchy.treemap)().size([width, height]).tile(tilingMethods[tile]).round(true).paddingInner(innerPadding).paddingOuter(outerPadding);

    var root = treemap((0, _d3Hierarchy.hierarchy)(_root).sum(value));

    var nodes = leavesOnly ? root.leaves() : root.descendants();

    return nodes.map(function (d) {
        d.color = color(_extends({}, d.data, { depth: d.depth }));

        d.data.key = d.ancestors().map(function (a) {
            return identity(a.data);
        }).join('.');

        return d;
    });
};