'use strict';

exports.__esModule = true;
exports.scalesFromObject = exports.createScale = exports.scalesPropType = exports.scalePropTypes = exports.scaleTypes = undefined;

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

var _d = require('d3');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var SCALE_LINEAR = 'linear';
var SCALE_BAND = 'band';
var SCALE_POINT = 'point';

var scaleTypes = exports.scaleTypes = [SCALE_LINEAR, SCALE_BAND, SCALE_POINT];

var scalePropTypes = exports.scalePropTypes = {
    type: _propTypes2.default.oneOf(scaleTypes).isRequired,
    axis: _propTypes2.default.oneOf(['x', 'y']).isRequired,
    dataKey: _propTypes2.default.oneOfType([_propTypes2.default.string, _propTypes2.default.number, _propTypes2.default.func]),
    aggregate: _propTypes2.default.array,
    maxOf: _propTypes2.default.array,
    padding: _propTypes2.default.number
};

var scalesPropType = exports.scalesPropType = function scalesPropType(props, propName, componentName) {
    var scales = props[propName];

    _lodash2.default.forOwn(scales, function (scale, id) {
        _propTypes2.default.checkPropTypes(scalePropTypes, scale, '`' + propName + '.' + id + '`', componentName);
    });
};

var createScale = exports.createScale = function createScale(_ref, data, width, height) {
    var type = _ref.type,
        axis = _ref.axis,
        key = _ref.key,
        aggregate = _ref.aggregate,
        maxOf = _ref.maxOf,
        padding = _ref.padding;

    var mapper = void 0;
    if (Array.isArray(aggregate)) {
        mapper = function mapper(d) {
            return _lodash2.default.sum(aggregate.map(function (k) {
                return d[k];
            }));
        };
    } else if (Array.isArray(maxOf)) {
        mapper = function mapper(d) {
            return _lodash2.default.max(maxOf.map(function (k) {
                return d[k];
            }));
        };
    } else if (_lodash2.default.isFunction(key)) {
        mapper = key;
    } else {
        mapper = function mapper(d) {
            return d[key];
        };
    }

    var range = [axis === 'y' ? height : 0, axis === 'x' ? width : 0];

    var scale = void 0;
    switch (type) {
        case SCALE_LINEAR:
            scale = (0, _d.scaleLinear)().rangeRound(range).domain([0, _lodash2.default.max(data.map(mapper))]);
            break;

        case SCALE_BAND:
            scale = (0, _d.scaleBand)().rangeRound(range).domain(data.map(mapper));

            if (padding !== undefined) {
                scale.padding(padding);
            }
            break;

        case SCALE_POINT:
            scale = (0, _d.scalePoint)().range(range).domain(data.map(mapper));
            break;
    }

    return scale;
};

var scalesFromObject = exports.scalesFromObject = function scalesFromObject(object, width, height, data) {
    return _lodash2.default.mapValues(object, function (scale, id) {
        return createScale(Object.assign({}, scale, { id: id }), data, width, height);
    });
};